#include <stdlib.h>
#include <memory.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <SDL.h>
#include <OWLIFTLib.h>


#define ARRAYSIZE(a) (sizeof(a) / sizeof((a)[0]))
#define ERRH	"***ERROR:"

typedef struct
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *darea;
	GtkWidget *lbl_fpatemp;
	GtkWidget *lbl_housingtemp;
	GtkWidget *lbl_timecounter;
	SDL_Surface *surface;
	SDL_Overlay *overlay;
	SDL_Rect orect;
	bool surface_set;
	guchar *img;
	int img_wx, img_wy;
	bool running;
	size_t img_size;
	pthread_mutex_t mutex;
	bool exit;
	const char *devPath;

	OwDev owDev;
} App;


static void PrintError0(OwStatus_t st, OwHostErrorCode_t he)
{
	fprintf(stderr, "error(%d) %s\n", (int)st, OwLib_GetErrorMessage(st));
	if (st == OWST_V4L2_ERROR) {
		fprintf(stderr, "host error=%d\n", (int)he);
	}
}


static void PrintError(OwStatus_t st, OwDev owDev)
{
	fprintf(stderr, "error(%d) %s\n", (int)st, OwLib_GetErrorMessage(st));
	if (st == OWST_V4L2_ERROR) {
		fprintf(stderr, "host error=%d\n", (int)OwLib_GetHostError(owDev));
	}
}


static void SetIntToLabel(GtkWidget *lbl, UINT32 v)
{
	char str[32];

	sprintf(str, "%u", v);
	gtk_label_set_text(GTK_LABEL(lbl), str);
}


static void SetDoubleToLabel(GtkWidget *lbl, double v)
{
	char str[32];

	sprintf(str, "%.2lf", v);
	gtk_label_set_text(GTK_LABEL(lbl), str);
}


static void RGB8888toYUY2(UINT8 *src, UINT8 *dst, UINT16 w, UINT16 h)
{
	int r1, g1, b1, r2, g2, b2;
	int y1, y2, u, v;
	UINT32 *rgb, *yuv, *yuve, p;

	rgb = (UINT32 *)src;
	yuv = (UINT32 *)dst;
	yuve = (UINT32 *)((UINT8 *)dst + ((w * h) << 1));

	while (yuv != yuve) {
		p = *rgb++;
		r1 = (p >>  0) & 0xFF;
		g1 = (p >>  8) & 0xFF;
		b1 = (p >> 16) & 0xFF;
		p = *rgb++;
		r2 = (p >>  0) & 0xFF;
		g2 = (p >>  8) & 0xFF;
		b2 = (p >> 16) & 0xFF;
		y1 = ((257 * r1) + (504 * g1) + (98 * b1)) / 1000 + 16;
		y2 = ((257 * r2) + (504 * g2) + (98 * b2)) / 1000 + 16;
		v =  ((-148 * r1) + (-291 * g1) + (439 * b1) +
			(-148 * r2) + (-291 * g2) + (439 * b2)) / 2 / 1000 + 128;
		u = ((439 * r1) + (-368 * g1) + (-71 * b1) +
			(439 * r2) + (-368 * g2) + (-71 * b2)) / 2 / 1000 + 128;
		*yuv++ = (y1 & 0xFF) | ((u & 0xFF) << 8) | ((y2 & 0xFF) << 16) | ((v & 0xFF) << 24);
	}
}


static void CaptureProc(OwDev owDev, BYTE *buf, UINT32 bufLen, void *userData)
{
	App *app = (App *)userData;
	LepTelemetryDataLineA *telem =
		(LepTelemetryDataLineA *)(buf + app->img_wx * app->img_wy * 2);
	UINT16 fpaTemp, housingTemp;
	UINT32 timeCounter;

	if (OwLib_Decode(owDev, buf, app->img, app->img_size, 0,
		OWDECODETYPE_AGC_RAINBOW_32) != OWST_OK) {
		memset(app->img, 0, app->img_size);
	}

	fpaTemp = VOSPI16_TO_UINT16(telem->fpaTemp_Kx100);
	housingTemp = VOSPI16_TO_UINT16(telem->housingTemp_Kx100);
	timeCounter = VOSPI32_TO_UINT32(telem->timeCounter);

	OwLib_FinishDecode(owDev);

	if (!app->exit && app->surface_set) {
		gdk_threads_enter();
		pthread_mutex_lock(&app->mutex);
		SDL_LockYUVOverlay(app->overlay);
		RGB8888toYUY2(app->img, (UINT8 *)app->overlay->pixels[0],
			app->img_wx, app->img_wy);
		SDL_UnlockYUVOverlay(app->overlay);
		SDL_DisplayYUVOverlay(app->overlay, &app->orect);

		SetDoubleToLabel(app->lbl_fpatemp, fpaTemp / 100.0);
		SetDoubleToLabel(app->lbl_housingtemp, housingTemp / 100.0);
		SetIntToLabel(app->lbl_timecounter, timeCounter);
		gdk_threads_leave();
		pthread_mutex_unlock(&app->mutex);
	}
}


static bool DeviceGet(App *app)
{
	OwDev *owDevs;
	int numDevs;
	OwStatus_t st;
	OwHostErrorCode_t he;

	if (app->devPath) {
		st = OwLib_GetDeviceByDeviceFile(app->devPath, &app->owDev, &he);
	} else {
		st = OwLib_GetDevices(&owDevs, &numDevs, &he);
		if (st == OWST_OK) {
			app->owDev = owDevs[0];
			OwLib_ReleaseArray(owDevs);
		}
	}

	if (st != OWST_OK) {
		PrintError0(st, he);
		return false;
	}

	if ((st = OwLib_CaptureSetup(app->owDev, OWFRAMERATE_8P6, CaptureProc,
		app)) != OWST_OK) {
		PrintError(st, app->owDev);
		return false;
	}

	app->img_wx = OwLib_GetFrameWidth(app->owDev);
	app->img_wy = OwLib_GetFrameHeight(app->owDev);

	return true;
}


static bool DeviceStart(App *app)
{
	OwStatus_t st;

	if (app->running) {
		return false;
	}

	if (app->owDev == NULL) {
		if (!DeviceGet(app))
			return false;
	}

	if ((st = OwLib_CaptureStart(app->owDev)) != OWST_OK) {
		PrintError(st, app->owDev);
		return false;
	}

	app->running = true;

	return true;
}


static void DeviceRelease(App *app)
{
	OwLib_Release(app->owDev);
	app->owDev = NULL;
}


static void* DeviceStopProc(void *arg)
{
	App *app = (App *)arg;

	if (app->owDev == NULL || !app->running)
		return NULL;

	OwLib_CaptureStop(app->owDev);
	DeviceRelease(app);
	app->running = false;

	return NULL;
}


static void DeviceStop(App *app)
{
	pthread_t stopThread;

	pthread_create(&stopThread, NULL, DeviceStopProc, app);
}


static gboolean On_delete_event(GtkWidget *widget,
	GdkEvent *event, gpointer udata)
{
	return FALSE;
}


static void AppExit(App *app)
{
	pthread_mutex_lock(&app->mutex);
	app->exit = true;
	pthread_mutex_unlock(&app->mutex);

	gtk_main_quit();

	DeviceStopProc(app);
	DeviceRelease(app);

	SDL_FreeYUVOverlay(app->overlay);
	SDL_FreeSurface(app->surface);
	free(app->img);
}


static void On_destroy(GtkWidget *widget, gpointer udata)
{
	App *app = (App *)udata;

	AppExit(app);
}


static void SDLSetWindow(GtkWidget *w)
{
	static char env[32];

	sprintf(env,"SDL_WINDOWID=%ld",
		(long)GDK_WINDOW_XID(w->window));
	putenv(env);
}


static gboolean On_configure(GtkWidget *widget,
	GdkEventConfigure *event, gpointer user_data)
{
	App *app = (App *)user_data;
	int w, h, bpp = 24;
	uint32_t vflags = SDL_ANYFORMAT | SDL_RESIZABLE;
	const SDL_VideoInfo *vinfo;
	GtkAllocation rec;

	gtk_widget_get_allocation(app->darea, &rec);
	w = rec.width;
	h = rec.height;

	if (app->surface == NULL) {
		SDLSetWindow(app->window);

		if (SDL_Init(SDL_INIT_VIDEO) < 0)  {
			fprintf(stderr, "SDL_Init failed\n");
			exit(1);
		}
	}

	pthread_mutex_lock(&app->mutex);

	if (app->surface) {
		if (app->orect.w == w &&
			app->orect.h == h)
			goto Exit;

		SDL_FreeSurface(app->surface);
		app->surface = NULL;
	}

	if (app->overlay) {
		SDL_FreeYUVOverlay(app->overlay);
		app->overlay = NULL;
	}

	vinfo = SDL_GetVideoInfo();

	if (vinfo == NULL) {
		fprintf(stderr, "SDL_GetVideoInfo failed\n");
		exit(1);
	}

	if (vinfo->hw_available) {
		vflags |= SDL_HWSURFACE;
		vflags |= SDL_DOUBLEBUF;
	} else {
		vflags |= SDL_SWSURFACE;
	}

	bpp = SDL_VideoModeOK(w, h, bpp, vflags);

	if (!bpp) {
		fprintf(stderr, "SDL_VideoModeOK failed\n");
		exit(1);
	}

	if ((app->surface = SDL_SetVideoMode(w, h, bpp, vflags)) == NULL) {
		fprintf(stderr, "SDL_SetVideoMode failed\n");
		exit(1);
	}

	if ((app->overlay = SDL_CreateYUVOverlay(app->img_wx, app->img_wy,
		SDL_YUY2_OVERLAY, app->surface)) == NULL) {
		fprintf(stderr, "SDL_CreateYUVOverlay failed\n");
		exit(1);
	}

	app->orect.x = 0;
	app->orect.y = 0;
	app->orect.w = w;
	app->orect.h = h;

	app->surface_set = TRUE;

Exit:

	gtk_widget_queue_draw(app->window);

	pthread_mutex_unlock(&app->mutex);

	return FALSE;
}


static void On_btn_start_clicked(GtkWidget *widget, gpointer udata)
{
	DeviceStart((App *)udata);
}


static void On_btn_stop_clicked(GtkWidget *widget, gpointer udata)
{
	DeviceStop((App *)udata);
}


static void On_btn_readffcmode_clicked(GtkWidget *widget, gpointer udata)
{
	App *app = (App *)udata;
	UINT16 data[16];
	OwStatus_t st;
	unsigned int i;

	if ((st = OwLib_CommandGet(app->owDev, 0x0200, 0x3C, data,
		ARRAYSIZE(data))) != OWST_OK) {
		PrintError(st, app->owDev);
		return;
	}

	printf("Read FFC Mode\n");

	for (i = 0; i < ARRAYSIZE(data); ++i) {
		printf("%04X ", data[i]);
	}

	printf("\n");
}


static void On_btn_runffc_clicked(GtkWidget *widget, gpointer udata)
{
	App *app = (App *)udata;
	OwStatus_t st;

	if ((st = OwLib_CommandRun(app->owDev, 0x0200, 0x40)) != OWST_OK) {
		PrintError(st, app->owDev);
		return;
	}
}


static void On_btn_exit_clicked(GtkWidget *widget, gpointer udata)
{
	On_destroy(widget, udata);
}


static void AppInit(App *app)
{
	memset(app, 0, sizeof(App));
	pthread_mutex_init(&app->mutex, NULL);
}

static void WindowInit(App *app)
{
	GtkWidget *btn_start, *btn_stop, *btn_readffcmode;
	GtkWidget *btn_runffc, *btn_exit;
	GtkWidget *lbl1, *lbl2, *lbl3;

	gtk_init(NULL, NULL);
	gdk_threads_init();
	gdk_threads_enter();
	app->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect(G_OBJECT(app->window), "delete_event",
		G_CALLBACK(On_delete_event), NULL);
	gtk_window_set_position(
		GTK_WINDOW(app->window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_window_resize(
		GTK_WINDOW(app->window), 550, 300);
	g_signal_connect(G_OBJECT(app->window), "destroy",
		G_CALLBACK(On_destroy), app);
	gtk_signal_connect(GTK_OBJECT(app->window), "configure-event",
		GTK_SIGNAL_FUNC(On_configure), (gpointer)app);
	gtk_container_set_border_width(GTK_CONTAINER(app->window), 0);

	app->hbox = gtk_hbox_new(false, 4);
	gtk_container_add(GTK_CONTAINER(app->window), app->hbox);

	app->darea = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(app->darea), 0, 0);
	gtk_widget_set_size_request(app->darea, app->img_wx * 2, app->img_wy * 2);

	app->vbox = gtk_vbox_new(true, 4);
	gtk_box_pack_start(GTK_BOX(app->hbox), app->darea, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(app->hbox), app->vbox, FALSE, FALSE, 0);

	btn_start = gtk_button_new_with_label("Start");
	g_signal_connect(GTK_OBJECT(btn_start), "clicked",
		G_CALLBACK(On_btn_start_clicked), (gpointer)app);

	btn_stop = gtk_button_new_with_label("Stop");
	g_signal_connect(GTK_OBJECT(btn_stop), "clicked",
		G_CALLBACK(On_btn_stop_clicked), (gpointer)app);

	btn_readffcmode = gtk_button_new_with_label("Read FFC Mode");
	g_signal_connect(GTK_OBJECT(btn_readffcmode), "clicked",
		G_CALLBACK(On_btn_readffcmode_clicked), (gpointer)app);

	btn_runffc = gtk_button_new_with_label("Run FFC");
	g_signal_connect(GTK_OBJECT(btn_runffc), "clicked",
		G_CALLBACK(On_btn_runffc_clicked), (gpointer)app);

	btn_exit = gtk_button_new_with_label("Exit");
	g_signal_connect(GTK_OBJECT(btn_exit), "clicked",
		G_CALLBACK(On_btn_exit_clicked), (gpointer)app);

	lbl1 = gtk_label_new_with_mnemonic("FPA Temp:");
	app->lbl_fpatemp = gtk_label_new_with_mnemonic("0");
	lbl2 = gtk_label_new_with_mnemonic("Housing Temp:");
	app->lbl_housingtemp = gtk_label_new_with_mnemonic("0");
	lbl3 = gtk_label_new_with_mnemonic("Time Counter:");
	app->lbl_timecounter = gtk_label_new_with_mnemonic("0");

	gtk_box_pack_start(GTK_BOX(app->vbox), btn_start, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), btn_stop, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), btn_readffcmode, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), btn_runffc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), btn_exit, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), lbl1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), app->lbl_fpatemp, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), lbl2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), app->lbl_housingtemp, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), lbl3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(app->vbox), app->lbl_timecounter, FALSE, FALSE, 0);

	app->img_size = app->img_wx * app->img_wy * 4;
	app->img = (guchar *)malloc(app->img_size);
	memset(app->img, 0xFF, app->img_size);

	gtk_widget_show_all(app->window);

	gdk_threads_leave();
}


static App *app_for_sigint_handler;

static void sigint_handler(int sig)
{
	App *app = app_for_sigint_handler;

	AppExit(app);

	fprintf(stderr, "exit\n");
	exit(0);
}


static void EventLoop(App *app)
{
	app_for_sigint_handler = app;
	signal(SIGINT, sigint_handler);

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();
}

static bool MatchStr(const char *s1, const char *s2, size_t *lenp)
{
	*lenp = strlen(s2);
	return strncmp(s1, s2, *lenp) == 0;
}

int main(int argc, char *argv[])
{
	App *app = (App *)malloc(sizeof(App));

	AppInit(app);

	for (int i = 1; i < argc; i++) {
		const char *arg = argv[i];
		size_t len;

		if (MatchStr(arg, "--dev=", &len)) {
			app->devPath = arg + len;
		} else {
			fprintf(stderr, ERRH "Unknown option. : %s\n", arg);
			return EXIT_FAILURE;
		}
	}

	if (!DeviceGet(app))
		return 1;
	WindowInit(app);
	EventLoop(app);

	free(app);

	return 0;
}

