﻿/**
 * @mainpage OWLIFT C Library
 * 
 * @~japanese
 * <ul>
 * <li>OWLIFTLib.h
 * <ul>
 * <li>関数
 * <ul>
 * <li><a href="group__Functions__Alloc.html">デバイスの取得と解放</a></li>
 * <li><a href="group__Functions__Prop.html">プロパティへのアクセス</a></li>
 * <li><a href="group__Functions__Sensor.html">センサ制御</a></li>
 * <li><a href="group__Functions__Image.html">画像の取得と変換</a></li>
 * <li><a href="group__Functions__File.html">ファイルへの記録と読込</a></li>
 * </ul>
 * <li><a href="group__Types.html">型</a></li>
 * <li><a href="group__Macros.html">マクロ</a></li>
 * </ul>
 * </li>
 * <li>OWLIFTDecoder.h</li>
 * <ul>
 * <li><a href="group__Decoder__GUID.html">GUID</a></li>
 * <li><a href="group__Decoder__IDL.html">IOWLIFTDecoder</a></li>
 * </ul>
 * </ul>
 *
 * @~english
 * <ul>
 * <li>OWLIFTLib.h
 * <ul>
 * <li>Functions
 * <ul>
 * <li><a href="group__Functions__Alloc.html">Getting and Releasing Device</a></li>
 * <li><a href="group__Functions__Prop.html">Accessing Properties</a></li>
 * <li><a href="group__Functions__Sensor.html">Controling Sensor</a></li>
 * <li><a href="group__Functions__Image.html">Getting and Converting Image</a></li>
 * <li><a href="group__Functions__File.html">Writing and Reading File</a></li>
 * </ul>
 * <li><a href="group__Types.html">Types</a></li>
 * <li><a href="group__Macros.html">Macros</a></li>
 * </ul>
 * <li>OWLIFTDecoder.h</li>
 * <ul>
 * <li><a href="group__Decoder__GUID.html">GUID</a></li>
 * <li><a href="group__Decoder__IDL.html">IOWLIFTDecoder</a></li>
 * </ul>
 * </ul>
 */

/**
 * @file OWLIFTLib.h
 */

#pragma once

 // 0xXXXXYYZZ => XXXX.YY.ZZ
#define OWLIB_VERSION					0x00010902
 // 0xXXYY => XX.YY
#define OWLIB_SUPPORTED_MIN_FW_VERSION	0x0100


#ifdef _WIN32
#include <Windows.h>
#endif

#ifdef __linux__
#include <stdint.h>
#ifndef INT8
#define INT8		int8_t
#endif
#ifndef UINT8
#define UINT8		uint8_t
#endif
#ifndef INT16
#define INT16		int16_t
#endif
#ifndef UINT16
#define UINT16		uint16_t
#endif
#ifndef INT32
#define INT32		int32_t
#endif
#ifndef UINT32
#define UINT32		uint32_t
#endif
#ifndef INT64
#define INT64		int64_t
#endif
#ifndef UINT64
#define UINT64		uint64_t
#endif
#ifndef BYTE
#define BYTE		uint8_t
#endif
#ifndef WORD
#define WORD		uint16_t
#endif
#ifndef DWORD
#define DWORD		uint32_t
#endif
#ifndef BOOL
#define BOOL		int
#endif
#ifndef WCHAR
#define WCHAR		wchar_t
#endif
#ifndef TRUE
#define TRUE		1
#endif
#ifndef FALSE
#define FALSE		0
#endif

#endif

#ifdef _WIN32
#ifdef OWLIFTLIB_EXPORTS
#define OWLIFTLIB_API __declspec(dllexport)
#else
#define OWLIFTLIB_API __declspec(dllimport)
#pragma comment(lib, "owlift.lib")
#endif
#else
#define OWLIFTLIB_API
#endif

//--------------------------------
// Macros
//

/**
 * @defgroup Macros Macros
 * @{
 */

// VOSPI 15:8  7:0
// Mac   7:0 15:8
// C     15:8  7:0

/**
 * @~japanese
 * デバイスが出力する16-bit値のバイトオーダをホストのエンディアンに変換します。
 *
 * @~english
 * Converts 16-bit value from device-byte order to host-byte order. 
 */
#define VOSPI16_TO_UINT16(v) \
	((((v) << 8) & 0xFF00) | ((((v) >> 8) & 0x00FF)))

/**
 * @~japanese
 * ホストの16-bit値のバイトオーダをデバイスのエンディアンに変換します。
 *
 * @~english
 * Converts 16-bit value from host-byte order to device-byte order. 
 */
#define UINT16_TO_VOSPI16(v) \
	((((v) << 8) & 0xFF00) | ((((v) >> 8) & 0xFF)))

// VOSPI 15:8  7:0   31:24 23:16
// Mac   23:16 31:24 7:0   15:8
// C     31:24 23:16 15:8  7:0

/**
 * @~japanese
 * デバイスが出力する32-bit値のバイトオーダをホストのエンディアンに変換します。
 *
 * @~english
 * Converts 32-bit value from device-byte order to host-byte order. 
 */
#define VOSPI32_TO_UINT32(v) \
	((((v) >> 8) & 0x000000FF) | (((v) << 8) & 0x0000FF00) | (((v) >> 8) & 0x00FF0000) | (((v) << 8) & 0xFF000000))

/**
 * @~japanese
 * ホストの32-bit値のバイトオーダをデバイスのエンディアンに変換します。
 *
 * @~english
 * Converts 32-bit value from host-byte order to device-byte order. 
 */
#define UINT32_TO_VOSPI32(v) \
	(((((v) & 0x000000FF) << 8) ) | ((((v) & 0x0000FF00) >> 8)) | ((((v) & 0x00FF0000) << 8)) | ((((v) & 0xFF000000) >> 8)))

/**
 * @~japanese
 * マニュアルゲイン制御における最低温度のデフォルト値。
 *
 * @~english
 * The default value of a minimum temperature for Manual Gain Control.
 */
#define OWDECODE_RANGE_MIN_TEMP_DEFAULT \
	(INT32)((273.15 + 16.0) * 100)

/**
 * @~japanese
 * マニュアルゲイン制御における最高温度のデフォルト値。
 *
 * @~english
 * The default value of a maximum temperature for Manual Gain Control.
 */
#define OWDECODE_RANGE_MAX_TEMP_DEFAULT \
	(INT32)((273.15 + 48.0) * 100)

/**
 * @~japanese
 * 自動ゲイン制御における最低温度のデフォルト値。
 *
 * @~english
 * The default value of a minimum temperature for Auto Gain Control.
 */
#define OWAGC_RANGE_MIN_TEMP_DEFAULT \
	(INT32)0

/**
 * @~japanese
 * 自動ゲイン制御における最高温度のデフォルト値。
 *
 * @~english
 * The default value of a maximum temperature for Auto Gain Control.
 */
#define OWAGC_RANGE_MAX_TEMP_DEFAULT \
	(INT32)((273.15 + 430) * 100)

/**
 * @~japanese
 * ノイズフィルタの閾値のデフォルト値。
 *
 * @~english
 * The default value of a threshold for Noise Filter.
 */
#define OWDECODE_NOISE_FILTER_THRESHOLD_DEFAULT		50
/**
 * @~japanese
 * ノイズフィルタの閾値の最小値。
 *
 * @~english
 * The minimum value of a threshold for Noise Filter.
 */
#define OWDECODE_NOISE_FILTER_THRESHOLD_MIN			1
/**
 * @~japanese
 * ノイズフィルタの閾値の最大値。
 *
 * @~english
 * The maximum value of a threshold for Noise Filter.
 */
#define OWDECODE_NOISE_FILTER_THRESHOLD_MAX			100

/**
 * @~japanese
 * シリアル番号のサイズ(WORD数)。
 *
 * @~english
 * The size of a serial number in WORDs.
 */
#define OWLIB_SERIAL_NUMBER_SIZE 4

/**
 * @~japanese
 * デバイスのコンフィグレーション: 再生時以外のときもセンサの電源をONにします（デフォルト値）。
 *
 * @~english
 * Device Configuration: Always turns on the sensor's power while not playing (default value).
 */
#define OWLIB_CONFIG_F_ALWAYS_ON	(1 << 0)

/**
 * @~japanese
 * デバイスのコンフィグレーション: 再生時以外のときもセンサの電源をONにします（デフォルト値）。
 *
 * @~english
 * Device Configuration: Always turns on the sensor's power while not playing (default value).
 */
#define OWLIB_CONFIG_F_ALWAYS_ON	(1 << 0)

/**
 * @~japanese
 * OWLIFT Type-AのProduct ID。
 *
 * @~english
 * OWLIFT Type-A's product ID.
 */
#define OWLIFT_TYPE_A_PID	0x2300

/**
 * @~japanese
 * OWLIFT Type-BのProduct ID。
 *
 * @~english
 * OWLIFT Type-B's product ID.
 */
#define OWLIFT_TYPE_B_PID	0x2301

/**
 * @~japanese
 * OWLIFT Type-FのProduct ID。
 *
 * @~english
 * OWLIFT Type-F's product ID.
 */
#define OWLIFT_TYPE_F_PID	0x2303

 /* @} */

//--------------------------------
// Types
//

/**
 * @defgroup Types Types
 * @{
 */

struct _OwDev;

/**
 * @~japanese
 * デバイスのハンドルを表します。
 *
 * @~english
 * Represents a device handle.
 */
typedef struct _OwDev *OwDev;

#ifdef _WIN32
/**
 * @~japanese
 * ホストのエラーコード(HRESULT)を表します。
 * HRESULT型については MSDN (Microsoft Developer Network) のドキュメントを参照してください。
 *
 * @~english
 * Represents a host error code (HRESULT).
 * See the documents of MSDN (Microsoft Developer Network) about HRESULT type.
 */
#endif

#ifdef __linux__
/**
 * @~japanese
 * ホストのエラーコード(Video4Linux の errno)を表します。
 * errno については Video4Linux のドキュメントを参照してください。
 *
 * @~english
 * Represents a host error code (errno in Video4Linux).
 * See the documents of Video4Linux about errno.
 */
#endif

typedef INT32 OwHostErrorCode_t;

/**
 * @~japanese
 * OwHostErrorCode_t の値において成功を表します。
 *
 * @~english
 * Represents success with OwHostErrorCode_t.
 */
#define OWHE_OK 0

/**
 * @~japanese
 * デバイスのエラーコードを表します。
 *
 * @~english
 * Represents a device error code.
 */
typedef unsigned char OwFirmwareErrorCode_t;

/**
 * @~japanese
 * ライブラリのエラーコードを表します。
 *
 * @~english
 * Represents a library error code.
 */
typedef enum
{
	/**
	 * @~japanese
	 * 成功。
	 *
	 * @~english
	 * Success.
	 */
	OWST_OK = 0,
	/**
	 * @~japanese
	 * ライブラリ内部エラー。
	 *
	 * @~english
	 * Library's internal error.
	 */
	OWST_INTERNAL_ERROR = 1,
	/**
	 * @~japanese
	 * メモリ不足。
	 *
	 * @~english
	 * Not enough memory.
	 */
	OWST_NO_MEMORY = 2,
	/**
	 * @~japanese
	 * デバイスが見つかりません。
	 *
	 * @~english
	 * No device was found.
	 */
	OWST_DEVICE_NOT_FOUND = 3,
	/**
	 * @~japanese
	 * デバイスの準備ができていません。
	 *
	 * @~english
	 * Device is not ready.
	 */
	OWST_DEVICE_NOT_READY = 4,
	/**
	 * @~japanese
	 * サポートされていないデバイスです。
	 *
	 * @~english
	 * Unsupported device.
	 */
	OWST_DEVICE_UNSUPPORTED = 5,
	/**
	 * @~japanese
	 * 領域のサイズが不足しています。
	 *
	 * @~english
	 * Not enough size of the area.
	 */
	OWST_NOT_ENOUGH_SIZE = 6,
	/**
	 * @~japanese
	 * 処理がまだ完了していません。
	 *
	 * @~english
	 * Processing is not completed.
	 */
	OWST_NOT_YET = 7,

	/**
	 * @~japanese
	 * デバイスのファームウェアでエラーが発生しました。
	 *
	 * @~english
	 * Error occurred in device firmware.
	 */
	OWST_FIRMWARE_ERROR = 10,
	/**
	 * @~japanese
	 * デバイスのセンサでエラーが発生しました。
	 *
	 * @~english
	 * Error occurred in device sensor.
	 */
	OWST_SENSOR_ERROR = 11,

#ifdef _WIN32
	/**
	 * @~japanese
	 * DirectShowがエラーコードを返しました。
	 *
	 * @~english
	 * Error occurred in DirectShow.
	 */
	OWST_DIRECTSHOW_ERROR = 12,
#endif

#ifdef __linux__
	/**
	 * @~japanese
	 * V4L2がエラーコードを返しました。
	 *
	 * @~english
	 * Error occurred in V4L2.
	 */
	OWST_V4L2_ERROR = 13,
#endif

	/**
	 * @~japanese
	 * 引数のポインタがNULLです。
	 *
	 * @~english
	 * Argument as a pointer type is NULL.
	 */
	OWST_NULL_POINTER = 20,
	/**
	 * @~japanese
	 * 引数の値が不正です。
	 *
	 * @~english
	 * Value of argument is invalid.
	 */
	OWST_INVALID_VALUE = 21,
	/**
	 * @~japanese
	 * 不正な状態で関数が実行されました。
	 *
	 * @~english
	 * Function is called in invalid state.
	 */
	OWST_INVALID_STATE = 22,
	/**
	 * @~japanese
	 * ファイルI/Oエラーが発生しました。
	 *
	 * @~english
	 * File I/O error occurred.
	 */
	OWST_FILE_IO_ERROR = 23,
	/**
	 * @~japanese
	 * 不正なファイルフォーマットです。
	 *
	 * @~english
	 * Invalid file format.
	 */
	OWST_INVALID_FILE_FORMAT = 24,
	/**
	 * @~japanese
	 * ファイルの終端です。
	 *
	 * @~english
	 * End of file.
	 */
	OWST_END_OF_FILE = 25,
	/**
	 * @~japanese
	 * 互換性がありません。
	 *
	 * @~english
	 * Incompatible.
	 */
	OWST_INCOMPATIBLE = 26,
} OwStatus_t;


/**
 * @~japanese
 * 赤外線センサのエラーコードを表します。
 * 定義されているのはエラーなしを表す定数のみですが、エラー発生時は"CCI/TWI Status Register"の"Response Error Code"を表します。
 * 詳しくは <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf">こちらのドキュメント</a> を参照してください。
 *
 * @~english
 * Represents an IR sensor error code.
 * This type defines only one consntant that means no-error,
 * but if an error occurred this type means "Response Error Code" in "CCI/TWI Status Register".
 * See <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf">this document</a> for detail.
 */
typedef enum
{
	/**
	 * @~japanese
	 * エラー無し。
	 *
	 * @~english
	 * No error.
	 */
	OW_SENSOR_ERROR_NONE = 0,
} OwSensorErrorCode_t;


/**
 * @~japanese
 * 画像出力のフレームレートを表します。
 *
 * @~english
 * Represents a frame rate for image output.
 */
typedef enum
{
	/**
	 * @~japanese
	 * デフォルト値 (8.6fps)
	 *
	 * @~english
	 * Default value (8.6fps)
	 */
	OWFRAMERATE_DEFAULT = 0,
	/**
	 * 8.6fps
	 */
	OWFRAMERATE_8P6 = 0,
	/**
	 * 25.9fps<br>
	 * @~japanese
	 * センサの性能は8.6fpsなので、このフレームレートを指定すると連続する3フレームは同一の内容になります。
	 * すなわち実質8.6fpsです。ただし、Telemetry Data は各フレームごとにアップデートされます。
	 * OWLIFT Type-A/B のみで有効です。
	 *
	 * @~english
	 * As the sensor's performance is 8.6 fps,
	 * 3 continuous frames are same data if 25.9 fps is specified.
	 * That is substantially 8.6 fps.
	 * However Telemetry Data is updated in every frame.
	 * This value works only for OWLIFT Type-A/B.
	 */
	OWFRAMERATE_25P9 = 1,
} OwFrameRate_t;


/**
 * @~japanese
 * 画像のデコード形式を表します。
 *
 * @~english
 * Represents the image decoding type.
 */
typedef enum
{
	/**
	 * @~japanese
	 * 未指定。
	 *
	 * @~english
	 * Not specified.
	 */
	OWDECODETYPE_UNKNOWN = -1,
	/**
	 * @~japanese
	 * Rawデータ（1ピクセル当たり2バイト・有効ビット数14、ビックエンディアン）のバイトオーダをホストのエンディアンに変換します。
	 *
	 * @~english
	 * Converts byte order of raw data to host endian.
	 * The format of raw data is 2 bytes per pixel, 14 effective bits, big-endian.
	 */
	OWDECODETYPE_RAW_14 = 0,
	/**
	 * @~japanese
	 * Rawデータを16-bit温度データに変換します。
	 *
	 * @~english
	 * Converts raw data to 16-bit temperature data.
	 */
	OWDECODETYPE_TEMPERATURE_16 = 1,
	/**
	 * @~japanese
	 * Rawデータを8bit グレースケールに変換します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 8bit gray scale data with Manual Gain Control.
	 */
	OWDECODETYPE_GRAY_8 = 10,
	/**
	 * @~japanese
	 * Rawデータを8bit グレースケールに変換します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 8bit gray scale data with Nonlinear Auto Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_GRAY_8 = 11,
	/**
	 * @~japanese
	 * OWDECODETYPE_NON_LINEAR_AGC_GRAY_8 と同じ意味です。
	 *
	 * @~english
	 * Same as OWDECODETYPE_NON_LINEAR_AGC_GRAY_8.
	 */
	OWDECODETYPE_AGC_GRAY_8 = 11,
	/**
	 * @~japanese
	 * Rawデータを8bit グレースケールに変換します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 8bit gray scale data with Linear Auto Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_GRAY_8 = 12,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Blue Orange" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_BLUE_ORANGE_24 = 20,
	/**
	 * @~japanese
	 * OWDECODETYPE_AGC_NON_LINEAR_BLUE_ORANGE_24 と同じ意味です。
	 *
	 * @~english
	 * Same as OWDECODETYPE_AGC_NON_LINEAR_BLUE_ORANGE_24.
	 */
	OWDECODETYPE_AGC_BLUE_ORANGE_24 = 21,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Blue Orange" color table
	 * and Nonlinear Auto Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_BLUE_ORANGE_24 = 21,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Blue Orange" color table
	 * and Linear Auto Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_BLUE_ORANGE_24 = 22,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Blue Orange" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_BLUE_ORANGE_32 = 30,
	/**
	 * @~japanese
	 * OWDECODETYPE_NON_LINEAR_AGC_BLUE_ORANGE_32 と同じ意味です。
	 *
	 * @~english
	 * Same as OWDECODETYPE_NON_LINEAR_AGC_BLUE_ORANGE_32.
	 */
	OWDECODETYPE_AGC_BLUE_ORANGE_32 = 31,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Blue Orange" color table
	 * and Nonlinear Auto Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_BLUE_ORANGE_32 = 31,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Blue Orange"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Blue Orange" color table
	 * and Linear Auto Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_BLUE_ORANGE_32 = 32,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Rainbow" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_RAINBOW_24 = 40,
	/**
	 * @~japanese
	 * OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_24 と同じ意味です。
	 *
	 * @~english
	 * Same as OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_24.
	 */
	OWDECODETYPE_AGC_RAINBOW_24 = 41,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Rainbow" color table
	 * and Nonlinear Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_24 = 41,
	/**
	 * @~japanese
	 * Rawデータを24-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 24-bit RGB data with "Rainbow" color table
	 * and Linear Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_RAINBOW_24 = 42,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Rainbow" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_RAINBOW_32 = 50,
	/**
	 * @~japanese
	 * OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_32 と同じ意味です。
	 *
	 * @~english
	 * Same as OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_32.
	 */
	OWDECODETYPE_AGC_RAINBOW_32 = 51,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Rainbow" color table
	 * and Nonlinear Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_32 = 51,
	/**
	 * @~japanese
	 * Rawデータを32-bit RGBに変換します。カラーテーブルに"Rainbow"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to 32-bit RGB data with "Rainbow" color table
	 * and Linear Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_RAINBOW_32 = 52,
	/**
	 * @~japanese
	 * Rawデータを RGB565 としてグレースケールに変換します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to gray scale data as RGB565 with Manual Gain Control.
	 */
	OWDECODETYPE_GRAY_16 = 60,
	/**
	 * @~japanese
	 * Rawデータを RGB565 としてグレースケールに変換します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to gray scale data as RGB565 with Nonlinear Auto Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_GRAY_16 = 61,
	/**
	 * @~japanese
	 * Rawデータを RGB565 としてグレースケールに変換します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to gray scale data as RGB565 with Linear Auto Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_GRAY_16 = 62,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Blue Orange"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Blue Orange" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_BLUE_ORANGE_16 = 70,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Blue Orange"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Blue Orange" color table
	 * and Nonlinear Auto Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_BLUE_ORANGE_16 = 71,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Blue Orange"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Blue Orange" color table
	 * and Linear Auto Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_BLUE_ORANGE_16 = 72,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Rainbow"を使用します。マニュアルゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Rainbow" color table
	 * and Manual Gain Control.
	 */
	OWDECODETYPE_RAINBOW_16 = 80,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Rainbow"を使用します。非線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Rainbow" color table
	 * and Nonlinear Gain Control.
	 */
	OWDECODETYPE_NON_LINEAR_AGC_RAINBOW_16 = 81,
	/**
	 * @~japanese
	 * Rawデータを RGB565 に変換します。カラーテーブルに"Rainbow"を使用します。線形型自動ゲイン制御を適用します。
	 *
	 * @~english
	 * Converts raw data to RGB565 data with "Rainbow" color table
	 * and Linear Gain Control.
	 */
	OWDECODETYPE_LINEAR_AGC_RAINBOW_16 = 82,
} OwDecodeType_t;

/**
 * @~japanese
 * 保護窓使用時の補正の種類を表します。
 *
 * @~english
 * Represents a correction type for protection windows.
 */
typedef enum {
	/**
	 * @~japanese
	 * 保護窓なし。
	 *
	 * @~english
	 * No protection window.
	 */
	OW_WINDOW_CORRECTION_TYPE_NONE,
	/**
	 * @~japanese
	 * 標準筐体 / GAT-05
	 *
	 * @~english
	 * Standard housing / GAT-05
	 */
	OW_WINDOW_CORRECTION_TYPE_GAT05_STD_HOUSING,
} OwWindowCorrectionType;

#ifdef _WIN32
#include <PshPack1.h>
#endif

#ifdef __linux__
#pragma pack(1)
#endif

/**
 * @~japanese
 * センサの出力に含まれる Telemetry Data を表します。
 * Telemetry Data の詳細については以下に示すセンサのデータシートを参照してください。
 * - <a href="http://www.infinitegra.co.jp/static/owlift/docs/Sensor.pdf" target="_blank">センサのデータシート</a>
 *
 * Telemetry Data はフレームデータの61ライン目（80x60x2バイト目）に格納されます。
 * センサのRawデータはビックエンディアンなので、
 * 構造体 LepTelemetryDataLineA によりマッピングした後にバイトオーダを変換する必要があります。
 * エンディアンの変換には以下のマクロを使用します。
 *
 * @~english
 * Represents Telemetry Data in sensor's output.
 * See the following documents for detail of Telemetry Data.
 * - <a href="http://www.infinitegra.co.jp/static/owlift/docs/Sensor.pdf" target="_blank">Sensor's Datasheet</a>
 *
 * Telemetry Data is stored from 61st line (80x60x2nd byte) in a frame data.
 * As the byte order of raw data is big-endian,
 * the byte order must be fixed after the Telemetry Data is mapped to structure LepTelemetryDataLineA.
 * Use following macros to fix it.
 *
 * @~
 * <ul>
 * <li>16-bit Integer (UINT16) ... VOSPI16_TO_UINT16(v)</li>
 * <li>32-bit Integer (UINT32) ... VOSPI32_TO_UINT32(v)</li>
 * </ul>
 */
typedef struct
{
	/** Line A offset  0 : Telemetry Revision */
	UINT16	telemetryRevision;
	/** Line A offset  1 : Time Counter */
	UINT32	timeCounter;
	union {
		/** Line A offset  3 : Status Bits */
		UINT32  statusBits;
		struct {
			/** Line A offset  3 b0..2 : Status Bits/reserved */
			UINT32 statusBits_reserved0					: 3;
			/** Line A offset  3 b3 : Status Bits/FFC designed */
			UINT32 statusBits_FFCDesired				: 1;
			/** Line A offset  3 b4..5 : Status Bits/FFC state */
			UINT32 statusBits_FFCState					: 2;
			/** Line A offset  3 b6..11 : Status Bits/reserved */
			UINT32 statusBits_reserved1					: 6;
			/** Line A offset  3 b12 : Status Bits/AGC state */
			UINT32 statusBits_AGCState					: 1;
			/** Line A offset  3 b13..14 : Status Bits/reserved */
			UINT32 statusBits_reserved2					: 2;
			/** Line A offset  3 b15 : Status Bits/reserved */
			UINT32 statusBits_shutterLockout			: 1;
			/** Line A offset  3 b16..19 : Status Bits/reserved */
			UINT32 statusBits_reserved3					: 4;
			/** Line A offset  3 b20 : Status Bits/reserved */
			UINT32 statusBits_OverTempShutdownImminent	: 1;
			/** Line A offset  3 b21..31 : Status Bits/reserved */
			UINT32 statusBits_reserved4					: 11;
		};
	};
	/** Line A offset  5 : Module Serial # */
	UINT16	moduleSerial[8];
	/** Line A offset 13 : Software Revision */
	UINT16	softwareRevision[4];
	/** Line A offset 17 : Reserved */
	UINT16	reservedA17[3];
	/** Line A offset 20 : Frame Counter */
	UINT32	frameCounter;
	/** Line A offset 22 : Frame Mean */
	UINT16	frameMean;
	/** Line A offset 23 : FPA Temp in counts */
	UINT16	fpaTemp_inCounts;
	/** Line A offset 24 : FPA Temp in Kelvin x 100 */
	UINT16	fpaTemp_Kx100;
	/** Line A offset 25 : Housing Temp in counts */
	UINT16	housingTemp_inCounts;
	/** Line A offset 26 : Housing Temp in kelvin x 100 */
	UINT16	housingTemp_Kx100;
	/** Line A offset 27 : Reserved */
	UINT16	reservedA27[2];
	/** Line A offset 29 : FPA Temp at last FFC */
	UINT16	fpaTempFFC_Kx100;
	/** Line A offset 30 : Time Counter at last FFC */
	UINT32	fpaTempFFC_timeCounter;
	/** Line A offset 32 : Housing Temp at last FFC */
	UINT16	housingTempFFC_Kx100;
	/** Line A offset 33 : Reserved */
	UINT16	reservedA33[1];
	/** Line A offset 34 : AGC ROI */
	UINT16	agcROI[4];
	/** Line A offset 38 : AGC Clip-Limit High */
	UINT16	agcClipLimitHigh;
	/** Line A offset 39 : AGC Clip-Limit Low */
	UINT16	agcClipLimitLow;
	/** Line A offset 40 : Reserved */
	UINT16	reservedA40[34];
	/** Line A offset 74 : Log2 of FFC frames */
	UINT16	log2OfFFCFrames;
	/** Line A offset 75 : Reserved */
	UINT16	reservedA75[5];

	/** Line B offset  0 : Reserved */
	UINT16	reservedB0[19];
	/** Line B offset 19 : Emissivity scaled by 8192 */
	UINT16	emissivity_x8192;
	/** Line B offset 20 : Background Temperature in kelvin x 100 */
	UINT16	backgroundTemp_Kx100;
	/** Line B offset 21 : Atmospheric Transmission scaled by 8192 */
	UINT16	atmosphericTrans_x8192;
	/** Line B offset 22 : Atmospheric Temperature in kelvin x 100 */
	UINT16	atmosphericTemp_Kx100;
	/** Line B offset 23 : Window Transmission scaled by 8192 */
	UINT16	windowTrans_x8192;
	/** Line B offset 24 : Window Reflection scaled by 8192 */
	UINT16	windowReflect_x8192;
	/** Line B offset 25 : Window Temperature in kelvin x 100 */
	UINT16	windowTemp_Kx100;
	/** Line B offset 26 : Window Reflected Temperature in kelvin x 100 */
	UINT16	windowReflectTemp_Kx100;
	/** Line B offset 27 : Reserved */
	UINT16	reservedB27[53];

	/** Line C offset  0 : Reserved */
	UINT16	reservedC0[5];
	/** Line C offset  5 : Gain Mode */
	UINT16	gainMode;
	/** Line C offset  6 : Effective Gain */
	UINT16	effectiveGain;
	/** Line C offset  7 : Gain Mode Desired Flag */
	UINT16	gainModeDesiredFlag;
	/** Line C offset  8 : Temperature Gain Mode Threshold High to Low (degC) */
	UINT16	tempGainModeThresholdHtoL_degC;
	/** Line C offset  9 : Temperature Gain Mode Threshold Low to High (degC)  */
	UINT16	tempGainModeThresholdLtoH_degC;
	/** Line C offset 10 : Temperature Gain Mode Threshold High to Low (K) */
	UINT16	tempGainModeThresholdHtoL_K;
	/** Line C offset 11 : Temperature Gain Mode Threshold Low to High (K) */
	UINT16	tempGainModeThresholdLtoH_K;
	/** Line C offset 12 : Reserved */
	UINT16	reservedC12[2];
	/** Line C offset 14 : Population Gain Mode Threshold High to Low */
	UINT16	populationGainModeThresholdHtoL;
	/** Line C offset 15 : Population Gain Mode Threshold Low to High */
	UINT16	populationGainModeThresholdLtoH;
	/** Line C offset 16 : Reserved */
	UINT16	reservedC16[6];
	/** Line C offset 22 : Gain Mode ROI */
	UINT16	gainModeROI[4];
	/** Line C offset 26 : Reserved */
	UINT16	reservedC26[22];
	/** Line C offset 48 : TLinear Enabled State */
	UINT16	tlinearEnabledState;
	/** Line C offset 49 : TLinear Resolution */
	UINT16	tlinearResolution;
	/** Line C offset 50 : Spotmeter Mean */
	UINT16	spotmeterMean;
	/** Line C offset 51 : Spotmeter Maximum */
	UINT16	spotmeterMax;
	/** Line C offset 52 : Spotmeter Minimum */
	UINT16	spotmeterMin;
	/** Line C offset 53 : Spotmeter Population */
	UINT16	spotmeterPopulation;
	/** Line C offset 54 : Spotmeter ROI Start Row */
	UINT16	spotmeterROIStartRow;
	/** Line C offset 55 : Spotmeter ROI Start Col */
	UINT16	spotmeterROIStartCol;
	/** Line C offset 56 : Spotmeter ROI End Row */
	UINT16	spotmeterROIEndRow;
	/** Line C offset 57 : Spotmeter ROI End Col */
	UINT16	spotmeterROIEndCol;
	/** Line C offset 58 : Reserved */
	UINT16	reservedC58[22];
} LepTelemetryDataLineA;


#ifdef _WIN32
#include <poppack.h>
#endif

#ifdef __linux__
#pragma pack()
#endif


/**
 * @~japanese
 * デバイス再生中に呼ばれるコールバック関数の型を表します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param buf [in]
 *		フレームバッファ。センサ出力のRawデータで、バイトオーダはビック・エンディアンです。
 *		OwLib_GetTelemetryData() に渡すことで、Telemetry Data へのポインタを取得可能です。
 * @param bufLen [in]
 *		buf が指す領域のサイズ（バイト数）。
 * @param userData [in]
 *		OwLib_CaptureSetup() に渡した userData。
 *
 * @~english
 * Represents a callback function type that is called while capturing.
 * @param dev [in]
 *		The device handle.
 * @param buf [in]
 *		The frame buffer. This is raw data of sensor output of which format is in big-endian.
 *		The pointer to Telemetry Data can be gotten by passing this value to OwLib_GetTelemetryData().
 * @param bufLen [in]
 *		The size of the area where 'buf' points in bytes.
 * @param userData [in]
 *		'userData' passed to OwLib_CaptureSetup().
 */
typedef void (*OwLibCaptureProc)(OwDev dev, BYTE *buf, UINT32 bufLen, void *userData);

/**
 * @~japanese
 * イベント通知のために呼ばれるコールバック関数の型を表します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param userData [in]
 *		コールバック関数を登録する際に一緒に渡した userData。
 *
 * @~english
 * Represents a callback function type that is called for notifications of events.
 * @param dev [in]
 *		The device handle.
 * @param userData [in]
 *		'userData' passed with the callback function at registering.
 */
typedef void (*OwLibNotifyProc)(OwDev dev, void *userData);


/**
 * @~japanese
 * 日時を表します。
 *
 * @~english
 * Represents a date time.
 */
typedef struct
{
	/**
	 * @~japanese
	 * 年 (2000..)
	 * @~english
	 * Year (2000..)
	 */
	WORD year;
	/**
	 * @~japanese
	 * 月 (1..12)
	 * @~english
	 * Month (1..12)
	 */
	WORD month;
	/**
	 * @~japanese
	 * 曜日 (0..6), 0=日曜日, 1=月曜日, ..
	 * @~english
	 * Day of week (0..6), 0=Sunday, 1=Monday, ..
	 */
	WORD dayOfWeek;
	/**
	 * @~japanese
	 * 日 (1..31)
	 * @~english
	 * Day (1..31)
	 */
	WORD day;
	/**
	 * @~japanese
	 * 時間 (0..23)
	 * @~english
	 * Hour (0..23)
	 */
	WORD hour;
	/**
	 * @~japanese
	 * 分 (0..59)
	 * @~english
	 * Minute (0..59)
	 */
	WORD minute;
	/**
	 * @~japanese
	 * 秒 (0..59)
	 * @~english
	 * Second (0..59)
	 */
	WORD second;
	/**
	 * @~japanese
	 * ミリ秒 (0..999)
	 * @~english
	 * Milli second 0..999
	 */
	WORD millisecond;
} OwDateTime;


/* @} */

//--------------------------------
// Functions
//

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/**
 * @defgroup Functions_Alloc  Getting and Releasing Device
 * @{
 */

////// Alloc/Release

#ifdef _WIN32
/**
 * @~japanese
 * デバイスを取得します。
 * デバイスが見つからないときは OWST_DEVICE_NOT_FOUND を返します。
 * デバイスのファームウェアのバージョンが不正なとき、OWST_DEVICE_UNSUPPORTED を返します。
 * OWST_DEVICE_UNSUPPORTED が返されたときは、無効な状態のデバイスハンドルが devsp に格納されます。
 * 無効な状態のデバイスハンドルに対しては、OwLib_GetFirmwareVersion() などのプロパティを取得する
 * いくつかの関数と、OwLib_Release() 以外は実行できません。
 * @param devsp [out]
 *		取得したデバイスハンドルが格納されます。
 *		*devsp の解放と、*devsp に格納された個々のデバイスハンドルの解放は呼び出し側の責任です。
 *		*devsp の解放は OwLib_ReleaseArray() によって行います。
 *		*devsp に格納された個々のデバイスハンドルの解放は OwLib_Release() によって行います。
 * @param numDevsp [out]
 *		取得したデバイスハンドルの個数が格納されます。
 * @param hep [out]
 *		戻り値が OWST_DIRECTSHOW_ERROR のとき、DirectShow のエラーコードが格納されます。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets devices.
 * Returns OWST_DEVICE_NOT_FOUND if not device was found.
 * Returns OWST_DEVICE_UNSUPPORTED if the firmware version is invalid.
 * If this function returns OWST_DEVICE_UNSUPPORTED,
 * a device handle with invalid state will be stored to devsp.
 * It cannot be executed against the device handle with invalid state except
 * OwLib_Release() and some functions that return properties.
 * @param devsp [out]
 *		The pointer that receives the device handle.
 *		Caller takes responsibility for releasing *devsp and each device handle stored in *devsp.
 *		OwLib_ReleaseArray() releases *devsp.
 *		OwLib_Release() releases each device handle stored in *devsp.
 * @param numDevsp [out]
 *		The pointer that receives the number of device handles.
 * @param hep [out]
 *		The pointer that receives the DirectShow error code
 *		when the return value is OWST_DIRECTSHOW_ERROR.
 * @return
 *		The error code.
 */
#endif

#ifdef __linux__
/**
 * @~japanese
 * デバイスを取得します。
 * デバイスが見つからないときは OWST_DEVICE_NOT_FOUND を返します。
 * デバイスのファームウェアのバージョンが不正なとき、OWST_DEVICE_UNSUPPORTED を返します。
 * OWST_DEVICE_UNSUPPORTED が返されたときは、無効な状態のデバイスハンドルが devsp に格納されます。
 * 無効な状態のデバイスハンドルに対しては、OwLib_GetFirmwareVersion() などのプロパティを取得する
 * いくつかの関数と、OwLib_Release() 以外は実行できません。
 * @param devsp [out]
 *		取得したデバイスハンドルが格納されます。
 *		*devsp の解放と、*devsp に格納された個々のデバイスハンドルの解放は呼び出し側の責任です。
 *		*devsp の解放は OwLib_ReleaseArray() によって行います。
 *		*devsp に格納された個々のデバイスハンドルの解放は OwLib_Release() によって行います。
 * @param numDevsp [out]
 *		取得したデバイスハンドルの個数が格納されます。
 * @param hep [out]
 *		戻り値が OWST_V4L2_ERROR のとき、V4L2 のエラーコードが格納されます。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets devices.
 * Returns OWST_EVICE_NOT_FOUND if not device was found.
 * Returns OWST_DEVICE_UNSUPPORTED if the firmware version is invalid.
 * If this function returns OWST_DEVICE_UNSUPPORTED,
 * a device handle with invalid state will be stored to devsp.
 * It cannot be executed against the device handle with invalid state except
 * OwLib_Release() and some functions that return properties.
 * @param devsp [out]
 *		The pointer that receives the device handle.
 *		Caller takes responsibility for releasing *devsp and each device handle stored in *devsp.
 *		OwLib_ReleaseArray() releases *devsp.
 *		OwLib_Release() releases each device handle stored in *devsp.
 * @param numDevsp [out]
 *		The pointer that receives the number of device handles.
 * @param hep [out]
 *		The pointer that receives the V4L2 error code
 *		when the return value is OWST_V4L2_ERROR.
 * @return
 *		The error code.
 */
#endif

OWLIFTLIB_API OwStatus_t
OwLib_GetDevices(OwDev **devsp, int *numDevsp, OwHostErrorCode_t *hep);

/**
 * @~japanese
 * 新しく接続したデバイスの取得と、切断したデバイスの検出を行います。
 * デバイスが見つからないときは OWST_DEVICE_NOT_FOUND を返します。
 * デバイスのファームウェアのバージョンが不正なとき、OWST_DEVICE_UNSUPPORTED を返します。
 * OWST_OK または OWST_DEVICE_NOT_FOUND または OWST_DEVICE_UNSUPPORTED が返されたとき、
 * newDevsp、 numNewDevsp、 removedDevsp、 numRemovedDevsp には値が格納されます。
 *
 * @param newDevsp [out]
 *		新しく接続したデバイスハンドルが格納されます。
 *		新しく接続したデバイスハンドルが存在しないときは NULL が格納されます。
 *		*newDevsp の解放と、*newDevsp に格納された個々のデバイスハンドルの解放は呼び出し側の責任です。
 *		*newDevsp の解放は OwLib_ReleaseArray() によって行います。
 *		*newDevsp に格納された個々のデバイスハンドルの解放は OwLib_Release() によって行います。
 * @param numNewDevsp [out]
 *		*newDevspに格納されたデバイスハンドルの個数が格納されます。
 * @param removedDevsp [out]
 *		oldDevs に格納されたデバイスハンドルのうち、切断したデバイスハンドルが格納されます。
 *		切断したデバイスハンドルが存在しないときは NULL が格納されます。
 *		*removedDevsp の解放と、*removedDevsp に格納された個々のデバイスハンドルの解放は呼び出し側の責任です。
 *		*removedDevsp の解放は OwLib_ReleaseArray() によって行います。
 * @param numRemovedDevsp [out]
 *		*removedDevspに格納されたデバイスハンドルの個数が格納されます。
 *		removedDevsp が NULL のときは NULL 指定可。
 * @param oldDevs [in]
 *		取得済みのデバイスハンドルの配列を指定します。
 *		NULL を指定した場合、切断の検知は行いません。
 * @param numOldDevs [in]
 *		oldDevs に格納されたデバイスハンドルの個数を指定します。
 * @param hep [out]
 *		OwLib_GetDevices() の hep を参照してください。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets newly connected devices and detects disconnected devices.
 * Returns OWST_DEVICE_NOT_FOUND if not device was found.
 * Returns OWST_DEVICE_UNSUPPORTED if the firmware version is invalid.
 * If this function returns OWST_OK, OWST_DEVICE_NOT_FOUND or OWST_DEVICE_UNSUPPORTED,
 * values will be stored into newDevsp, numNewDevsp, removedDevsp and numRemovedDevsp.
 *
 * @param newDevsp [out]
 *		The pointer that receives the newly connected device handle.
 *		If no device was newly connected, returns NULL.
 *		Caller takes responsibility for releasing *newDevspp and each device handle stored in *newDevspp.
 *		OwLib_ReleaseArray() releases *newDevsp.
 *		OwLib_Release() releases each device handle stored in *newDevsp.
 * @param numNewDevsp [out]
 *		The pointer that receives the number of device handles stored into newDevsp.
 * @param removedDevsp [out]
 *		The pointer that receives the disconnected devices within oldDevs.
 *		If no device was detected as disconnected, returns NULL.
 *		Caller takes responsibility for releasing *removedDevsp.
 *		OwLib_ReleaseArray() releases *removedDevsp.
 * @param numRemovedDevsp [out]
 *		The pointer that receives the number of device handles stored into removedDevsp.
 * @param oldDevs [in]
 *		The device handles that are already connected.
 *		If NULL is specified, disconnected devices will not be detected.
 * @param numOldDevs [in]
 *		The number of device handles stored into oldDevs.
 * @param hep [out]
 *		See hep in OwLib_GetDevices().
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetDevicesDiff(OwDev **newDevsp, int *numNewDevsp,
	OwDev **removedDevsp, int *numRemovedDevsp,
	OwDev *oldDevs, int numOldDevs, OwHostErrorCode_t *hep);

#ifdef _WIN32

/**
 * @~japanese
 * 指定の DirectShow のキャプチャフィルタに対応するデバイスハンドルを取得します。
 * キャプチャフィルタが操作可能なデバイスではないときは、OWST_DEVICE_NOT_FOUND を返します。 
 * @param baseFilter [in]
 *		IBaseFilter*型のキャプチャフィルタを表すオブジェクト。 
 * @param devp [out]
 *		デバイスハンドルが格納されます。
 * @param hep [out]
 *		戻り値が OWST_DIRECTSHOW_ERROR のとき、DirectShow のエラーコードが格納されます。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets a device handle corresponding to the specified DirectShow capture filter.
 * Returns OWST_DEVICE_NOT_FOUND if the capture filter is an inoperable one.
 * @param baseFilter [in]
 *		The object that represents a capture filter of which type is IBaseFilter*.
 * @param devp [out]
 *		The pointer that receives a device handle.
 * @param hep [out]
 *		The pointer that receives the DirectShow error code
 *		when this function returns OWST_DIRECTSHOW_ERROR.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetDeviceByCaptureFilter(void *baseFilter, OwDev *devp,
	OwHostErrorCode_t *hep);

#endif

#ifdef __linux__

/**
 * @~japanese
 * 指定のビデオデバイスに対応するデバイスハンドルを取得します。
 * ビデオデバイスが操作可能なデバイスではないときは、OWST_DEVICE_NOT_FOUND を返します。 
 * @param devPath [in]
 *		プリフィックスが /dev/video のデバイスファイルのパス。
 * @param devp [out]
 *		デバイスハンドルが格納されます。
 * @param hep [out]
 *		戻り値が OWST_V4L2_ERROR のとき、V4L2 のエラーコードが格納されます。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets a device handle corresponding to the specified video device.
 * Returns OWST_DEVICE_NOT_FOUND if the video device is an inoperable one.
 * @param devPath [in]
 *		The device file path of which prefix is /dev/video.
 * @param devp [out]
 *		The pointer that receives a device handle.
 * @param hep [out]
 *		The pointer that receives the V4L2 error code
 *		when this function returns OWST_V4L2_ERROR.
 * @return
 *		The error code.
 */
OwStatus_t OwLib_GetDeviceByDeviceFile(const char *devPath, OwDev *devp,
	OwHostErrorCode_t *hep);

#endif

/**
 * @~japanese
 * デバイスと関連付かない OwDev オブジェクトを生成します。
 * 生成したオブジェクトは OWLIFT SDK 経由ではない DirectShow や V4L2 をソースとした
 * 画像の取得を行った場合に、OwLib_Decode() を呼び出すためだけに使うことができます。
 * @param devp [out]
 *		オブジェクトの格納先。
 * @return
 *		エラーコード。
 *
 * @~english
 * Creates a new OwDev object that is not related with any devices.
 * The created object can ONLY be used for calling OwLib_Decode()
 * if the image source is directly DirectShow or V4L2 without OWLIFT SDK.
 * @param devp [out]
 *		The pointer that receives an object.
 * @return
 *		The error code.
 */
OwStatus_t OwLib_CreateWithoutConnect(OwDev *devp);

/**
 * @~japanese
 * デバイスハンドルを解放します。
 * @param dev [in]
 *		デバイスハンドル。
 *
 * @~english
 * Releases a device handle.
 * @param dev [in]
 *		The device handle.
 */
OWLIFTLIB_API void
OwLib_Release(OwDev dev);

/**
 * @~japanese
 * デバイスハンドルの配列を解放します。配列に格納された各デバイスハンドルは解放されません。
 * @param devs [in]
 *		デバイスハンドルの配列。
 *
 * @~english
 * Releases an array of device handle.
 * Each device handle stored in the array is not released.
 * @param devs [in]
 *		The array of device handles.
 */
OWLIFTLIB_API void
OwLib_ReleaseArray(OwDev *devs);

/**
 * @~japanese
 * ソフトウェア上でデバイスをホストから一旦切断し、再接続します。
 * Type-F においてはファームウェアバージョン 3.1 以降で使用可能です。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Disconnect the device from the host and reconnect it by software operation.
 * Type-F supports this function only after the firmware version 3.1. 
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_Reconnect(OwDev dev);

/* @} */

////// Properties

/**
 * @defgroup Functions_Prop  Accessing Properties
 * @{
 */

/**
 * @~japanese
 * ライブラリのバージョンを取得します。
 * @return
 *		該当する値。バージョン番号がX.Y.Zのとき、b31..16 がX、b15..b8 がY、b7..b0 がZです。
 *
 * @~english
 * Gets the library version number.
 * @return
 *		The corresponding value.
 *		If the version number is X.Y.Z,
 *		X is assigned to b31..16, Y is assigned to b15..b8 and Z is assigned b7..b0.
 */
OWLIFTLIB_API UINT32
OwLib_GetLibVersion(void);

/**
 * @~japanese
 * サポートするデバイスファームウェアの最低バージョンを取得します。
 * @return
 *		該当する値。バージョン番号がX.Yのとき、b15..b8 がX、b7..b0 がYです。
 *
 * @~english
 * Gets the minimum supported version number of the device firmware.
 * @return
 *		The corresponding value.
 *		If the version number is X.Y
 *		X is assigned to b15..8, Y is assigned to b7..b0.
 */
OWLIFTLIB_API UINT16
OwLib_GetSupportedFWVersion(void);

/**
 * @~japanese
 * USBデバイスとしての Vendor ID (idVendor) を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the Vendor ID (idVendor) as an USB device.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetVendorID(OwDev dev);

/**
 * @~japanese
 * USBデバイスとしての Product ID (idProduct) を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the Product ID (idProduct) as an USB device.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetProductID(OwDev dev);

/**
 * @~japanese
 * USBデバイスとしての Release Number (bcdDevice) を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the Release Number (bcdDevice) as an USB device.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetReleaseNumber(OwDev dev);

/**
 * @~japanese
 * デバイスファームウェアのバージョンを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。バージョン番号がX.Yのとき、b15..b8 がX、b7..b0 がYです。
 *
 * @~english
 * Gets the device firmware version number.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 *		If the version number is X.Y
 *		X is assigned to b15..8, Y is assigned to b7..b0.
 */
OWLIFTLIB_API UINT16
OwLib_GetFirmwareVersion(OwDev dev);

/**
 * @~japanese
 * センサモジュールのシリアル番号を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param serialp [out]
 *		シリアル番号が格納されます。serialp の配列サイズは OWLIB_SERIAL_NUMBER_SIZE 以上である必要があります。
 *
 * @~english
 * Gets the serial number of the sensor module.
 * @param dev [in]
 *		The device handle.
 * @param serialp [out]
 *		The pointer that receives the serial number.
 *		The array size of serialp must be greater than or equal to OWLIB_SERIAL_NUMBER_SIZE.
 */
OWLIFTLIB_API void
OwLib_GetSerialNumber(OwDev dev, UINT16 *serialp);

/**
 * @~japanese
 * センサモジュールのシリアル番号を UINT64 型で取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		シリアル番号。
 *
 * @~english
 * Gets the serial number of the sensor module as UINT64 type.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The serial number.
 */
OWLIFTLIB_API UINT64
OwLib_GetSerialNumberAsUINT64(OwDev dev);

/**
 * @~japanese
 * フレームの横の解像度を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the horizontal resolution of the frame.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetFrameWidth(OwDev dev);

/**
 * @~japanese
 * フレームの縦の解像度を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the vertical resolution of the frame.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetFrameHeight(OwDev dev);

/**
 * @~japanese
 * フレームレートの分母を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the denominator of the frame rate.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API int
OwLib_GetFrameRateDenominator(OwDev dev);

/**
 * @~japanese
 * フレームレートの分子を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the numerator of the frame rate.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API int
OwLib_GetFrameRateNumerator(OwDev dev);

/**
 * @~japanese
 * 円周方向の温度補正の有効・無効の状態を取得します。
 * OWLIFT Type-A のみで有効です。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns whether or not the temperature correction of circumferential direction is enabled.
 * This function only works with OWLIFT Type-A.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetCircumferenceCorrection(OwDev dev);

/**
 * @~japanese
 * 円周方向の温度補正の有効・無効の状態を設定します。
 * OWLIFT Type-A のみで有効です。
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Enables or disables the temperature correction of circumferential direction.
 * This function only works with OWLIFT Type-A.
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API void
OwLib_SetCircumferenceCorrection(OwDev dev, BOOL enabled);

/**
 * @~japanese
 * ノイズフィルタの有効・無効の状態を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns whether or not the Noise Filter is enabled.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetNoiseFilter(OwDev dev);

/**
 * @~japanese
 * ノイズフィルタの有効・無効の状態を設定します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Enables or disables the Noise Filter.
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API void
OwLib_SetNoiseFilter(OwDev dev, BOOL enabled);

/**
 * @~japanese
 * ノイズフィルタの閾値を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the threshold of the Noise Filter.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetNoiseFilterThreshold(OwDev dev);

/**
 * @~japanese
 * ノイズフィルタの閾値を設定します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param threshold [in]
 *		OWDECODE_NOISE_FILTER_THRESHOLD_MIN から OWDECODE_NOISE_FILTER_THRESHOLD_MAX の範囲の値。
 *		OWDECODE_NOISE_FILTER_THRESHOLD_MIN のとき最も補正の強度が小さいです。
 *
 * @~english
 * Sets the threshold of the Noise Filter.
 * @param dev [in]
 *		The device handle.
 * @param threshold [in]
 *		The value from OWDECODE_NOISE_FILTER_THRESHOLD_MIN to OWDECODE_NOISE_FILTER_THRESHOLD_MAX.
 *		OWDECODE_NOISE_FILTER_THRESHOLD_MIN is minimum correction strength.
 */
OWLIFTLIB_API void
OwLib_SetNoiseFilterThreshold(OwDev dev, UINT16 threshold);

/**
 * @~japanese
 * 歪み補正の有効・無効の状態を取得します。
 * デフォルト値は OWLIFT Type-A/B のとき FALSE、OWLIFT Type-F のとき TRUE です。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns whether or not the Undistortion is enabled.
 * The default value is FALSE for OWLIFT Type-A/B, is TRUE for OWLIFT Type-F.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetUndistortion(OwDev dev);

/**
 * @~japanese
 * 歪み補正の有効・無効の状態を設定します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Enables or disables the Undistortion.
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API void
OwLib_SetUndistortion(OwDev dev, BOOL enabled);

/**
 * @~japanese
 * 最後に実行した関数で発生したホストのエラーコードを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the host error code occurred in the function that was called last.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API OwHostErrorCode_t
OwLib_GetHostError(OwDev dev);

/**
 * @~japanese
 * 最後に実行した関数で発生したデバイスのエラーコードを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the device error code occurred in the function that was called last.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API OwFirmwareErrorCode_t
OwLib_GetFirmwareError(OwDev dev);

/**
 * @~japanese
 * 最後に実行した関数で発生したセンサのエラーコードを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets the sensor error code occurred in the function that was called last.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API OwSensorErrorCode_t
OwLib_GetSensorError(OwDev dev);

/**
 * @~japanese
 * デバイスが切断されたかどうかを取得します。
 * 制限事項として、OwLib_GetDeviceByCaptureFilter() によって取得したデバイスに対しては
 * 常に FALSE が返されることに注意してください。
 * 
 * @~english
 * Gets whether or not the device was disconnected.
 * As a restriction, this function always returns FALSE if the device was
 * gotten by OwLib_GetDeviceByCaptureFilter().
 */
OWLIFTLIB_API BOOL
OwLib_GetDisconnected(OwDev dev);

/**
 * @~japanese
 * エラーコードに対応するメッセージを取得します。
 * @param st [in]
 *		ライブラリのエラーコード。
 * @return
 *		エラーメッセージ。
 *
 * @~english
 * Gets the corresponding messsage to the error code.
 * @param st [in]
 *		The library error code.
 * @return
 *		The error message.
 */
OWLIFTLIB_API const char*
OwLib_GetErrorMessage(OwStatus_t st);

#ifdef _WIN32
/**
 * @~japanese
 * デバイスハンドルに関連付くDirectShowキャプチャフィルタを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		IBaseFilter*型のオブジェクト。
 *
 * @~english
 * Gets the DirectShow capture filter related to the device handle.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The object that represents a capture filter of which type is IBaseFilter*.
 */
OWLIFTLIB_API void*
OwLib_GetCaptureFilter(OwDev dev);
#endif

/**
 * @~japanese
 * デバイスのコンフィグレーションを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param cfgp [out]
 *		デバイスのコンフィグレーションが格納されます。値は以下に示すフラグ群です。
 *		<ul>
 *			<li>OWLIB_CONFIG_F_ALWAYS_ON
 *		</ul>
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets the device configuration.
 * @param dev [in]
 *		The device handle.
 * @param cfgp [out]
 *		The pointer that receives the device configuration.
 *		The value is constructed with following flags.
 *		<ul>
 *			<li>OWLIB_CONFIG_F_ALWAYS_ON
 *		</ul>
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetConfig(OwDev dev, UINT16 *cfgp);

/**
 * @~japanese
 * デバイスのコンフィグレーションを設定します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param cfg [in]
 *		デバイスのコンフィグレーション。値については OwLib_GetConfig() を参照してください。
 * @return
 *		エラーコード。
 *
 * @~english
 * Sets the device configuration.
 * @param dev [in]
 *		The device handle.
 * @param cfg [in]
 *		The device configuration value.
 *		See OwLib_GetCaptureFilter() about the value.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_SetConfig(OwDev dev, UINT16 cfg);

/**
 * @~japanese
 * デバイスが再生中かどうかを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param activep [out]
 *		再生状態が格納されます。 TRUE : 再生中, FALSE : それ以外。
 * @return
 *		エラーコード。
 *
 * @~english
 * Test if the device is capturing.
 * @param dev [in]
 *		The device handle.
 * @param activep [out]
 * 		The pointer that receives whether or not the device is capturing.
 *		TRUE : The device is capturing, FALSE : otherwise.
 * @return
 *		The error code.
 *
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetActive(OwDev dev, BOOL *activep);

/**
 * @~japanese
 * フレームデータの熱画像の末尾に付くライン数を取得します。
 * Telemetry Dataのライン数を含みます。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the number of trailing lines after a thermography block in a frame data.
 * This includes the number of lines of Telemetry Data.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetFooterLines(OwDev dev);

/**
 * @~japanese
 * フレームデータの熱画像の末尾に付くTelemetry Dataのライン数を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the number of trailing lines which represent Telemetry Data
 * after a thermography block in a frame data.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetTelemetryDataLines(OwDev dev);


/**
 * @~japanese
 * 接続中のデバイスが持つTelemetry Dataのリビジョンを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns a revision number of the telemetry data that the connected device has.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API UINT16
OwLib_GetSupportedTelemetryDataRevision(OwDev dev);


/**
 * @~japanese
 * 実デバイスと関連したデバイスハンドルかどうかを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns whether or not the specified handle is related to a real device.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_IsRealDevice(OwDev dev);


/* @} */

/**
 * @defgroup Functions_Sensor  Controling Sensor
 * @{
 */

/**
 * @~japanese
 * センサのコマンドをコマンドタイプ"GET"で実行し、結果のレジスタを読み込みます。
 * センサのコマンドについては <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">こちらのドキュメント</a> を参照してください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param moduleID [in]
 *		Module ID。
 * @param cmdIDBase [in]
 *		Command ID Base。
 * @param data [out]
 *		レジスタの値が格納される配列。配列サイズが ndata 以上である必要があります。
 * @param ndata [in]
 *		読み込むレジスタのWORD数。
 *		読み込み可能なレジスタは DATA 0 Register .. DATA 15 Register であり、
 *		ndata は DATA 0 Register を先頭として何個のレジスタを読み込むかを意味します。
 *		バイトオーダはホストのエンディアンです。
 * @return
 *		エラーコード。
 *
 * @~english
 * Executes the sensor's command as command type "GET", and reads registers as a result.
 * See <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">this document</a> about sensor's commands.
 * @param dev [in]
 *		The device handle.
 * @param moduleID [in]
 *		The Module ID.
 * @param cmdIDBase [in]
 *		The Command ID Base.
 * @param data [out]
 * 		The array that receives registers' values.
 * 		The array size must be greater than or equal to 'ndata'.
 * @param ndata [in]
 * 		The number of the registers to read in WORDs.
 *		DATA 0 Register .. DATA 15 Register are readable.
 *		'ndata' means the number of registers to read counting from DATA 0 Register.
 *		The byte order is host endian.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CommandGet(OwDev dev, UINT16 moduleID, UINT16 cmdIDBase,
	UINT16 *data, UINT16 ndata);

/**
 * @~japanese
 * センサのレジスタに指定の値を書き込み、センサのコマンドをコマンドタイプ"SET"で実行します。
 * センサのコマンドについては <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">こちらのドキュメント</a> を参照してください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param moduleID [in]
 *		Module ID。
 * @param cmdIDBase [in]
 *		Command ID Base。
 * @param data [in]
 *		レジスタの値を示す配列。配列サイズが ndata 以上である必要があります。
 * @param ndata [in]
 *		書き込むレジスタのWORD数。
 *		書込み可能なレジスタは DATA 0 Register .. DATA 15 Register であり、
 *		ndata は DATA 0 Register を先頭として何個のレジスタを書き込むかを意味します。
 *		バイトオーダはホストのエンディアンです。
 * @return
 *		エラーコード。
 *
 * @~english
 * Writes the specified value to the register, and executes the sensor's command as command type "SET".
 * See <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">this document</a> about sensor's commands.
 * @param dev [in]
 *		The device handle.
 * @param moduleID [in]
 *		The Module ID.
 * @param cmdIDBase [in]
 *		The Command ID Base.
 * @param data [in]
 * 		The array that stores registers' values.
 * 		The array size must be greater than or equal to 'ndata'.
 * @param ndata [in]
 * 		The number of the registers to write in WORDs.
 *		DATA 0 Register .. DATA 15 Register are writable.
 *		'ndata' means the number of registers to write counting from DATA 0 Register.
 *		The byte order is host endian.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CommandSet(OwDev dev, UINT16 moduleID, UINT16 cmdIDBase,
	const UINT16 *data, UINT16 ndata);

/**
 * @~japanese
 * センサののコマンドをコマンドタイプ"RUN"で実行します。
 * センサのコマンドについては <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">こちらのドキュメント</a> を参照してください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param moduleID [in]
 *		Module ID。
 * @param cmdIDBase [in]
 *		Command ID Base。
 * @return
 *		エラーコード。
 *
 * @~english
 * Executes the sensor's command as command type "RUN".
 * See <a href="http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf" target="_blank">this document</a> about sensor's commands.
 * @param dev [in]
 *		The device handle.
 * @param moduleID [in]
 *		The Module ID.
 * @param cmdIDBase [in]
 *		The Command ID Base.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CommandRun(OwDev dev, UINT16 moduleID, UINT16 cmdIDBase);

/* @} */

/**
 * @defgroup Functions_Image  Getting and Converting Image
 * @{
 */

/**
 * @~japanese
 * 再生の設定を行います。この関数を実行すると内部的に再生用のデータが確保されます。
 * 再生を開始・終了した後、確保された領域を解放するには OwLib_CaptureRelease() を実行します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param frameRate [in]
 *		フレームレート。
 * @param proc [in]
 *		再生中に呼ばれるコールバック関数。デバイスからフレームが出力されるたびに呼ばれます。
 *		ユーザは OwLibCaptureProc 型のユーザ定義関数の中でフレームを取得します。
 * @param userData [in]
 *		proc に渡す任意の値。
 * @return
 *		エラーコード。
 *
 * @~english
 * Configures the capturing.
 * This function allocates data for capturing.
 * Caller takes responsibility to release the allocated data by OwLib_CaptureRelease()
 * after stop capturing.
 * @param dev [in]
 *		The device handle.
 * @param frameRate [in]
 *		The frame rate.
 * @param proc [in]
 * 		The callback function being called while capturing.
 * 		This function is called every time a frame is retrieved from the device.
 * 		The user can take frames in the user-defined function of which type is OwLibCaptureProc.
 * @param userData [in]
 *		The arbitrary value passed to 'proc'.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CaptureSetup(OwDev dev, OwFrameRate_t frameRate, OwLibCaptureProc proc, void *userData);

/**
 * @~japanese
 * 再生を開始します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Starts capturing.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CaptureStart(OwDev dev);

/**
 * @~japanese
 * 再生を終了します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Stops capturing.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_CaptureStop(OwDev dev);

/**
 * @~japanese
 * 再生の設定を解放します。
 * @param dev [in]
 *		デバイスハンドル。
 *
 * @~english
 * Releases a configuration data for capturing.
 * @param dev [in]
 *		The device handle.
 *
 */
OWLIFTLIB_API void
OwLib_CaptureRelease(OwDev dev);

/**
 * @~japanese
 * 指定のフレームバッファ上の Telemetry Data へのポインタを取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @param buf [in]
 *		フレームバッファ。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets the pointer to the Telemetry Data in the specified frame buffer.
 * @param dev [in]
 *		The device handle.
 * @param buf [in]
 *		The frame buffer.
 * @return
 *		The error code.
 */
OWLIFTLIB_API LepTelemetryDataLineA*
OwLib_GetTelemetryData(OwDev dev, BYTE *buf);

/**
 * @~japanese
 * フレームのRawデータを各種の形式および温度データに変換します。
 * 原則的に OwLib_CaptureSetup() に渡す OwLibCaptureProc 型のコールバック関数内で実行してください。
 * 1つのフレームに対してこの関数を1回以上実行した場合、次のフレームに対してこの関数の実行する前に、
 * OwLib_FinishDecode() を実行する必要があります。
 * OwLib_CaptureStart() の後、一定時間は OWST_OK 以外を返す可能性があります。
 * OWST_OK 以外が返された場合は、コールバック関数を抜けてください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param srcBuf [in]
 *		フレームのRawデータ。
 * @param dstBuf [out]
 *		変換後のフレームを格納するバッファ。
 * @param dstBufLen [in]
 *		dstBuf が指す領域のサイズ。バイト単位。
 * @param dstBufStride [in]
 *		dstBuf が指す領域のストライド。バイト単位。
 *		0 を指定すると自動的に計算されます。
 *		dtype が OWDECODETYPE_RAW_14, OWDECODETYPE_TEMPERATURE_16 のときは無効です。
 * @param dtype [in]
 *		デコード形式。
 * @return
 *		エラーコード。
 *
 * @~english
 * Converts raw data to data of other formats or temperature data.
 * In principle, call this function in the OwLibCaptureProc callback function
 * passed to OwLib_CaptureSetup().
 * If this function is called once or more times against a frame,
 * OwLib_FinishDecode() must be called before next calling.
 * After calling OwLib_CaptureStart(), there is a possibility that this function
 * returns except OWST_OK while a fixed term.
 * If this function returns except OWST_OK, return from your callback function.
 * @param dev [in]
 *		The device handle.
 * @param srcBuf [in]
 *		The raw data of the frame.
 * @param dstBuf [out]
 *		The buffer into which 'srcBuf' is converted.
 * @param dstBufLen [in]
 *		The size of the area where 'dstBuf' points in bytes.
 * @param dstBufStride [in]
 *		The stride of the area where 'dstBuf' points in bytes.
 *		If 0 is specified, the value is automatically calculated.
 *		This parameter is not used when a'dtype' is OWDECODETYPE_RAW_14 or OWDECODETYPE_TEMPERATURE_16.
 * @param dtype [in]
 *		The decoding type.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_Decode(OwDev dev, BYTE *srcBuf, BYTE *dstBuf, UINT32 dstBufLen,
	UINT32 dstBufStride, OwDecodeType_t dtype);

/**
 * @~japanese
 * 1つのフレーム対する OwLib_Decode() の実行を終了したことを通知します。
 * @param dev [in]
 *		デバイスハンドル。
 *
 * @~english
 * Notifies the completion for calling OwLib_Decode() against a frame.
 * @param dev [in]
 *		The device handle.
 */
OWLIFTLIB_API void
OwLib_FinishDecode(OwDev dev);

/**
 * @~japanese
 * 温度データを取得します。
 * 1つのフレームに対して1回以上 OwLib_Decode() を実行した後から OwLib_FinishDecode() を実行するまでの間、取得可能です。
 * OwLib_Decode() で dtype に OWDECODETYPE_TEMPERATURE_16 を指定した場合と同じデータが取得されます。
 * @param dev [in]
 *		デバイスハンドル。
 * @param buf [out]
 *		温度データが格納されます。1ピクセルあたり16-bitです。
 *		単位は OwLib_GetLastTempResolution() によって取得される分解能で決まります。
 *		- OWLIFT Type-A の温度分解能は 1/100。温度データを100で割ると Kelvin 値が得られます。
 *		- OWLIFT Type-B, F の温度分解能は 1/10。温度データを10で割ると Kelvin 値が得られます。
 * @param bufLen [in]
 *		buf が指す領域のサイズ（バイト数）。
 * @return
 *		エラーコード。
 *
 * @~english
 * Gets the temperature data.
 * The data can be gotten between calling OwLib_Decode() once or more times against a frame
 * and calling OwLib_FinishDecode().
 * The data is same as one that can be gotten by OwLib_Decode()
 * with passing 'dtype' as OWDECODETYPE_TEMPERATURE_16.
 * @param dev [in]
 *		The device handle.
 * @param buf [out]
 *		The pointer that receives the temperature data.
 *		The format is 16-bit per pixel.
 *		The unit is represented by OwLib_GetLastTempResolution().
 *		- The temperature resolution of OWLIFT Type-A is 1/100. A kelvin value is obtained by dividing a temperature data by 100.
 *		- The temperature resolution of OWLIFT Type-B and F is 1/10. A kelvin value is obtained by dividing a temperature data by 10.
 * @param bufLen [in]
 *		The size of the area where 'buf' points in bytes.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetTempTable(OwDev dev, UINT16 *buf, UINT32 bufLen);

/**
 * @~japanese
 * 最後にデコードしたフレームにおける温度テーブルの分解能を取得します。
 * OwLib_GetTempTable() の温度データをこの値で割るとKelvin値が得られます。
 * この関数は呼ぶのは OwLib_Decode() を呼んだ後である必要があります。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		10, 100
 *
 * @~english
 * Returns the resolution of the temperature table corresponding to the last decoded frame.
 * A kelvin value is obtained by dividing a temperature value of OwLib_GetTempTable() by this value.
 * This function must not be called before calling OwLib_Decode().
 * @param dev [in]
 *		The device handle.
 * @return
 *		10, 100
 */
OWLIFTLIB_API int
OwLib_GetLastTempResolution(OwDev dev);

/**
 * @~japanese
 * マニュアルゲイン制御の画像変換について、最低温度と最高温度を取得します。
 * マニュアルゲイン制御では、指定の最低温度がカラーテーブルの最小値、最高温度がカラーテーブルの最大値となるよう補正します。
 * 最低温度未満の温度は最低温度の色に、最高温度以上の温度は最高温度の色に切り詰められます。
 * @param dev [in]
 *		デバイスハンドル。
 * @param minTempp [out]
 *		最低温度が格納されます。単位は 0.01 Kelvin。
 * @param maxTempp [out]
 *		最高温度が格納されます。単位は 0.01 Kelvin。
 *
 * @~english
 * Gets the minimum and maximum temperature for Manual Gain Control.
 * Manual Gain Control corrects colors of image so that
 * the specified minimum temperature becomes the minimum color value in the color table
 * and the maximum temperature becomes the maximum color value.
 * The temperature less than the specified minimum temperature is corrected to
 * the minimum color value.
 * The temperature greater than the specified maximum temperature is corrected to
 * the maximum color value.
 * @param dev [in]
 *		The deivce handle.
 * @param minTempp [out]
 * 		The pointer that receives the minimum temperature. The unit is 0.01 Kelvin.
 * @param maxTempp [out]
 * 		The pointer that receives the maximum temperature. The unit is 0.01 Kelvin.
 */
OWLIFTLIB_API void
OwLib_GetDecodeRange(OwDev dev, INT32 *minTempp, INT32 *maxTempp);

/**
 * @~japanese
 * マニュアルゲイン制御の画像変換について、最低温度と最高温度を設定します。
 * マニュアルゲイン制御の動作については OwLib_GetDecodeRange() も合わせて参照してください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param minTemp [in]
 *		最低温度。単位は 0.01 Kelvin。
 * @param maxTemp [in]
 *		最高温度。単位は 0.01 Kelvin。
 *
 * @~english
 * Sets the minimum and maximum temperature for Manual Gain Control.
 * See OwLib_GetDecodeRange() about Manual Gain Control.
 * @param dev [in]
 *		The device handle.
 * @param minTemp [in]
 * 		The minimum temperature. The unit is 0.01 Kelvin.
 * @param maxTemp [in]
 * 		The maximum temperature. The unit is 0.01 Kelvin.
 */
OWLIFTLIB_API void
OwLib_SetDecodeRange(OwDev dev, INT32 minTemp, INT32 maxTemp);

/**
 * @~japanese
 * 自動ゲイン制御の画像変換について、最低温度と最高温度を取得します。
 * 最低温度未満の温度は最低温度の色に、最高温度以上の温度は最高温度の色に切り詰められます。
 * @param dev [in]
 *		デバイスハンドル。
 * @param minTempp [out]
 *		最低温度が格納されます。単位は 0.01 Kelvin。
 * @param maxTempp [out]
 *		最高温度が格納されます。単位は 0.01 Kelvin。
 *
 * @~english
 * Gets the minimum and maximum temperature for Auto Gain Control.
 * The temperature less than the specified minimum temperature is corrected to
 * the minimum color value.
 * The temperature greater than the specified maximum temperature is corrected to
 * the maximum color value.
 * @param dev [in]
 *		The deivce handle.
 * @param minTempp [out]
 * 		The pointer that receives the minimum temperature. The unit is 0.01 Kelvin.
 * @param maxTempp [out]
 * 		The pointer that receives the maximum temperature. The unit is 0.01 Kelvin.
 */
OWLIFTLIB_API void
OwLib_GetAGCRange(OwDev dev, INT32 *minTempp, INT32 *maxTempp);

/**
 * @~japanese
 * 自動ゲイン制御の画像変換について、最低温度と最高温度を設定します。
 * 自動ゲイン制御の動作については OwLib_GetAGCRange() も合わせて参照してください。
 * @param dev [in]
 *		デバイスハンドル。
 * @param minTemp [in]
 *		最低温度。単位は 0.01 Kelvin。
 * @param maxTemp [in]
 *		最高温度。単位は 0.01 Kelvin。
 *
 * @~english
 * Sets the minimum and maximum temperature for Auto Gain Control.
 * See OwLib_GetAGCRange() about Auto Gain Control.
 * @param dev [in]
 *		The device handle.
 * @param minTemp [in]
 * 		The minimum temperature. The unit is 0.01 Kelvin.
 * @param maxTemp [in]
 * 		The maximum temperature. The unit is 0.01 Kelvin.
 */
OWLIFTLIB_API void
OwLib_SetAGCRange(OwDev dev, INT32 minTemp, INT32 maxTemp);

/**
 * @~japanese
 * 指定のゲイン制御のデコード形式をマニュアルゲイン制御の形式に変換します。
 * @param dtype
 *		デコード形式。
 * @return
 *		dtype に対応するマニュアルゲイン制御の形式。
 *		dtype がマニュアルゲイン制御の形式のときは、dtype を返します。
 *		対応するマニュアルゲイン制御の形式がないときは OWDECODETYPE_UNKNOWN を返します。
 *
 * @~english
 * Converts the specified decoding type to the type of Manual Gain Control.
 * @param dtype
 *		The decoding type.
 * @return
 * 		The decoding type of Manual Gain Control corresponding to 'dtype'.
 * 		Returns 'dtype' if 'dtype' is the type of Manual Gain Control.
 * 		Returns OWDECODETYPE_UNKNOWN if the corresponding type is not found.
 */
OWLIFTLIB_API OwDecodeType_t
OwLib_GetManualGainType(OwDecodeType_t dtype);

/**
 * @~japanese
 * 指定のゲイン制御のデコード形式を線形型自動ゲイン制御の形式に変換します。
 * @param dtype
 *		デコード形式。
 * @return
 *		dtype に対応する線形型自動ゲイン制御の形式。
 *		dtype が線形型自動ゲイン制御の形式のときは、dtype を返します。
 *		対応する自動ゲイン制御の形式がないときは OWDECODETYPE_UNKNOWN を返します。
 *
 * @~english
 * Converts the specified decoding type to the type of Linear Auto Gain Control.
 * @param dtype
 *		The decoding type.
 * @return
 * 		The decoding type of Linear Auto Gain Control corresponding to 'dtype'.
 * 		Returns 'dtype' if 'dtype' is the type of Linear Gain Control.
 * 		Returns OWDECODETYPE_UNKNOWN if the corresponding type is not found.
 */
OWLIFTLIB_API OwDecodeType_t
OwLib_GetLinearAutoGainType(OwDecodeType_t dtype);

/**
 * @~japanese
 * 指定のマニュアルゲイン制御のデコード形式を非線形型自動ゲイン制御の形式に変換します。
 * @param dtype
 *		デコード形式。
 * @return
 *		dtype に対応する非線形型自動ゲイン制御の形式。
 *		dtype が非線形型自動ゲイン制御の形式のときは、dtype を返します。
 *		対応する自動ゲイン制御の形式がないときは OWDECODETYPE_UNKNOWN を返します。
 *
 * @~english
 * Converts the specified decoding type to the type of Nonlinear Auto Gain Control.
 * @param dtype
 *		The decoding type.
 * @return
 * 		The decoding type of Nonlinear Auto Gain Control corresponding to 'dtype'.
 * 		Returns 'dtype' if 'dtype' is the type of Nonlinear Gain Control.
 * 		Returns OWDECODETYPE_UNKNOWN if the corresponding type is not found.
 */
OWLIFTLIB_API OwDecodeType_t
OwLib_GetNonLinearAutoGainType(OwDecodeType_t dtype);

/**
 * @~japanese
 * OwLib_GetNonLinearAutoGainType() の別名です。
 * 詳細は OwLib_GetNonLinearAutoGainType() を参照してください。
 *
 * @~english
 * The alias of OwLib_GetNonLinearAutoGainType().
 * See OwLib_GetNonLinearAutoGainType() for detail.
 */
OWLIFTLIB_API OwDecodeType_t
OwLib_GetAutoGainType(OwDecodeType_t dtype);

/**
 * @~japanese
 * 温度データ差分出力の有効・無効の状態を取得します。
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets whether or not the Temperature Data Subtraction Output is enabled.
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetSubtractionTemp(OwDev dev);

/**
 * @~japanese
 * 温度データ差分出力の有効・無効の状態を設定します。
 * 有効に設定すると、現在の温度データから OwLib_SetSubtractionTempTable() によって設定された温度データが差し引かれます。
 * OwLib_GetTempTable() で取得される温度データは、差し引かれた後の値となります。
 * また、デコード処理は差し引かれた後の温度データを元に計算されます。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Enables or disables the Temperature Data Subtraction Output.
 * If enabled, the temperature data set by OwLib_SetSubtractionTempTable() is
 * subtracted from the current temperature data.
 * The temperature data got by OwLib_GetTempTable() will be subtracted data.
 * The decoding process will be done by subtracted data.
 *
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API void
OwLib_SetSubtractionTemp(OwDev dev, BOOL enabled);

/**
 * @~japanese
 * 温度データ差分出力の基準となる温度データを設定します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param buf [in]
 *		温度データ。
 * @param bufLen [in]
 *		buf が指す領域のサイズ（バイト数）。
 * @return
 *		エラーコード。
 *
 * @~english
 * Sets the reference temperature data for the Temperature Data Subtraction Output.
 *
 * @param dev [in]
 *		The device handle.
 * @param buf [in]
 *		The temperature data.
 * @param bufLen [in]
 *		The size of the area where 'buf' points in bytes.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_SetSubtractionTempTable(OwDev dev, UINT16 *buf, UINT32 bufLen);

/**
 * @~japanese
 * Rawデータから直接色変換を行うかどうかを設定します。
 * TRUE のとき温度を取得できません。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Returns whether or not colorizing by raw data directly.
 * If TRUE, you can not get temperature data. 
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API BOOL
OwLib_GetColorizeByRaw(OwDev dev);

/**
 * @~japanese
 * Rawデータから直接色変換を行うかどうかを取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		TRUE : 有効, FALSE: 無効。
 *
 * @~english
 * Returns whether or not colorizing by raw data directly.
 *
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		TRUE : enabled, FALSE: disabled.
 */
OWLIFTLIB_API void
OwLib_SetColorizeByRaw(OwDev dev, BOOL enabled);

/**
 * @~japanese
 * 保護窓使用時の補正の種類を設定します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param wctype [in]
 *		補正の種類。
 * @param reserved [in]
 *		未使用。
 *
 * @~english
 * Sets a correction type for protection windows.
 *
 * @param dev [in]
 *		The device handle.
 * @param wctype [in]
 *		The correction type.
 * @param reserved [in]
 *		Unused.
 */
OWLIFTLIB_API void
OwLib_SetWindowCorrection(OwDev dev, OwWindowCorrectionType wctype, int reserved);

/**
 * @~japanese
 * 保護窓使用時の補正の種類を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param wctype [out]
 *		補正の種類の格納先。
 * @param reserved [out]
 *		未使用。
 *
 * @~english
 * Gets a correction type for protection windows.
 *
 * @param dev [in]
 *		The device handle.
 * @param wctype [out]
 *		The pointer that receives the correction type.
 * @param reserved [out]
 *		Unused.
 */
OWLIFTLIB_API void
OwLib_GetWindowCorrection(OwDev dev, OwWindowCorrectionType *wctype, int *reserved);


/**
 * @~japanese
 * 反射補正データファイルを設定します。
 * 反射補正は保護窓に反射するOWLIFTの像を軽減します。
 * 反射補正データファイルは OWLIFTCap で生成してエクスポートしてください。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param filePath [in]
 *		反射補正データファイルのパス。
 * @return
 *		エラーコード。
 *
 * @~english
 * Sets a reflection correction data file.
 * Reflection correction reduce a reflected image of OWLIFT in a protection window.
 * Reflection correction data file can be generated and exported by OWLIFTCap.
 *
 * @param dev [in]
 *		The device handle.
 * @param filePath [in]
 *		The file path of a reflection correction data file.
 * @return
 *		The error code.
 */
#ifdef _WIN32
OWLIFTLIB_API OwStatus_t
OwLib_SetReflectCorrFile(OwDev dev, const wchar_t *filePath);
#else
OWLIFTLIB_API OwStatus_t
OwLib_SetReflectCorrFile(OwDev dev, const char *filePath);
#endif

/**
 * @~japanese
 * High Gain モードに切り替えます。
 * Type-A では使用できません。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Enables High Gain mode.
 * Cannot use against Type-A.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_EnableHighGainMode(OwDev dev);

/* @} */

/**
 * @defgroup Functions_File  Writing and Reading File
 * @{
 */

/**
 * @~japanese
 * Raw録画の設定を行います。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param filePath [in]
 *		Raw録画ファイルのパス。
 * @return
 *		エラーコード。
 *
 * @~english
 * Configures the raw-recording.
 *
 * @param dev [in]
 *		The device handle.
 * @param filePath [in]
 *		The file path of a raw-recording file.
 * @return
 *		The error code.
 */
#ifdef _WIN32
OWLIFTLIB_API OwStatus_t
OwLib_FileWriteSetup(OwDev dev, const wchar_t *filePath);
#else
OWLIFTLIB_API OwStatus_t
OwLib_FileWriteSetup(OwDev dev, const char *filePath);
#endif

/**
 * @~japanese
 * Raw録画を開始します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Starts the raw-recording.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_FileWriteStart(OwDev dev);

/**
 * @~japanese
 * Raw録画を停止します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		エラーコード。
 *
 * @~english
 * Stops the raw-recording.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_FileWriteStop(OwDev dev);

/**
 * @~japanese
 * パッシブモードのRaw録画を設定します。
 * この関数はフレームを保存するタイミングを任意に変更したいときに用います。
 * OwLib_FileWritePassiveAdd() と共に使用します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param filePath [in]
 *		Raw録画ファイルのパス。
 * @param frameRateNumerator [in]
 *		フレームレートの分子。
 * @param frameRateDenominator [in]
 *		フレームレートの分母。
 * @return
 *		エラーコード。
 *
 * @~english
 * Starts the raw-recording.
 * This function is used when it is needed to change the timing to save frames.
 * Use this with OwLib_FileWritePassiveAdd().
 *
 * @param dev [in]
 *		The device handle.
 * @param filePath [in]
 *		The file path of a raw-recording file.
 * @param frameRateNumerator [in]
 *		The numerator of the frame rate.
 * @param frameRateDenominator [in]
 *		The denominator of the frame rate.
 * @return
 *		The error code.
 */
#ifdef _WIN32
OWLIFTLIB_API OwStatus_t
OwLib_FileWritePassiveSetup(OwDev dev, const wchar_t *filePath,
	int frameRateNumerator, int frameRateDenominator);
#else
OWLIFTLIB_API OwStatus_t
OwLib_FileWritePassiveSetup(OwDev dev, const char *filePath,
	int frameRateNumerator, int frameRateDenominator);
#endif

/**
 * @~japanese
 * Raw録画ファイルへフレームを追加します。
 * この関数は OwLib_FileWritePassiveSetup() を呼んだ場合に限り用います。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param buf [in]
 *		フレームバッファ。センサ出力のRawデータで、バイトオーダはビック・エンディアンです。
 * @param bufLen [in]
 *		buf が指す領域のサイズ（バイト数）。
 * @return
 *		エラーコード。
 *
 * @~english
 * Add a frame to the raw-recording file.
 * Call this function when you called OwLib_FileWritePassiveSetup().
 *
 * @param dev [in]
 *		The device handle.
 * @param buf [in]
 *		The frame buffer. This is raw data of sensor output of which format is in big-endian.
 * @param bufLen [in]
 *		The size of the area where 'buf' points in bytes.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_FileWritePassiveAdd(OwDev dev, BYTE *buf, UINT32 bufLen);

/**
 * @~japanese
 * Raw再生の設定を行います。
 * Raw再生を行うには、最初にこの関数で再生用のデバイスハンドルを取得します。
 * この関数で取得されるデバイスハンドルは実際のデバイスのかわりに、ファイルをデータソースとします。
 * この関数で取得されるデバイスハンドルに対して、 OwLib_CaptureStart() を呼ぶことで再生を開始します。
 *
 * @param devp [out]
 *		取得したデバイスハンドルが格納されます。
 *		*devp の解放は呼び出し側の責任です。
 *		*devp の解放は OwLib_Release() によって行います。
 * @param filePath [in]
 *		Raw録画ファイルのパス。
 * @param capProc [in]
 *		再生中に呼ばれるコールバック関数。ファイルからフレームが取得されるたびに呼ばれます。
 *		ユーザは OwLibCaptureProc 型のユーザ定義関数の中でフレームを取得します。
 * @param endNotifyProc [in]
 *		ファイルの終端に達したときに呼ばれるコールバック関数。
 * @param userData [in]
 *		capProc と endNotifyProc に渡す任意の値。
 * @return
 *		エラーコード。
 *
 * @~english
 * Configures the raw-playing.
 * To play a raw-recording file, get the device handle for playing by this function.
 * The device handle this function returns uses a file as the data source instead of a real device.
 * To start playing, call OwLib_CaptureStart() against the device handle this function returns.
 *
 * @param devp [out]
 *		The pointer that receives a device handle.
 *		Caller takes responsibility for releasing *devp.
 *		OwLib_Release() releases *devp.
 * @param filePath [in]
 *		The file path of a raw-recording file.
 * @param capProc [in]
 * 		The callback function being called while playing.
 * 		This function is called every time a frame is retrieved from the file.
 * 		The user can take frames in the user-defined function of which type is OwLibCaptureProc.
 * @param endNotifyProc [in]
 *		The callback function being called at the end of file.
 * @param userData [in]
 *		The arbitrary value passed to 'capProc' and 'endNotifyProc'.
 * @return
 *		The error code.
 */
#ifdef _WIN32
OWLIFTLIB_API OwStatus_t
OwLib_FileReadSetup(OwDev *devp, const wchar_t *filePath,
	OwLibCaptureProc capProc, OwLibNotifyProc endNotifyProc, void *userData);
#else
OWLIFTLIB_API OwStatus_t
OwLib_FileReadSetup(OwDev *devp, const char *filePath,
	OwLibCaptureProc capProc, OwLibNotifyProc endNotifyProc, void *userData);
#endif

/**
 * @~japanese
 * Raw再生を一時停止します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 *
 * @~english
 * Pauses the raw-playing.
 *
 * @param dev [in]
 *		The device handle.
 */
OWLIFTLIB_API void
OwLib_FileReadPause(OwDev dev);

/**
 * @~japanese
 * Raw再生を再開します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 *
 * @~english
 * Resumes the raw-playing.
 *
 * @param dev [in]
 *		The device handle.
 */
OWLIFTLIB_API void
OwLib_FileReadResume(OwDev dev);

/**
 * @~japanese
 * Raw再生の位置を移動します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param timeOfs [in]
 *		先頭からの経過時間(ミリ秒)。
 * @return
 *		エラーコード。
 *
 * @~english
 * Resumes the raw-playing.
 *
 * @param dev [in]
 *		The device handle.
 * @param timeOfs [in]
 *		The time offset from the head in milli seconds.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_FileReadSeek(OwDev dev, UINT64 timeOfs);

/**
 * @~japanese
 * Raw録画ファイルの時間長を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the time length of the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API UINT64
OwLib_GetFileTimeLength(OwDev dev);


/**
 * @~japanese
 * Raw録画ファイルの最初のフレームのオフセット時間を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the time offset of the first frame in the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API UINT64
OwLib_GetFileHeadTimeOffset(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルから最後に読み込んだフレームの時間オフセットを取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the time offset of the frame last read from the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API UINT64
OwLib_GetFileCurrentTimeOffset(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルのフレーム数を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the number of frames of the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API UINT32
OwLib_GetFileFrames(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルから最後に読み込んだフレームの番号を取得します。
 * 先頭のフレームの番号は1です。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the frame number of the frame last read from the raw-recording file.
 * The first frame number is 1.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API UINT32
OwLib_GetFileCurrentFrameNumber(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルの最初のフレームの時間を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param dateTimep [out]
 * 		時間が格納されます。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the time of the first frame in the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @param dateTimep [out]
 * 		The pointer that receives the time.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API void
OwLib_GetFileHeadDateTime(OwDev dev, OwDateTime *dateTimep);

/**
 * @~japanese
 * Raw録画ファイルから最後に読み込んだフレームの時間を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param dateTimep [out]
 * 		時間が格納されます。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the time of the frame last read from the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @param dateTimep [out]
 * 		The pointer that receives the time.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API void
OwLib_GetFileCurrentDateTime(OwDev dev, OwDateTime *dateTimep);

/**
 * @~japanese
 * Raw録画ファイルの再生速度の倍率を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the scaling factor for the playback speed of raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API double
OwLib_GetFileReadSpeed(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルの再生速度の倍率を設定します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param speed [in]
 *		該当する値。0以上の値に限ります。0 は最高速度を意味します。
 *
 * @~english
 * Sets the scaling factor for the playback speed of raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @param speed [in]
 *		The correspoinding value that is greater than or equal to zero.
 *		Zero means maximum speed.
 */
OWLIFTLIB_API void
OwLib_SetFileReadSpeed(OwDev dev, double speed);


/**
 * @~japanese
 * Raw録画ファイルの再生を一時停止しているかを取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns whether or not playing raw-recording file is paused.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetFileReadPaused(OwDev dev);

/**
 * @~japanese
 * Raw録画ファイルから一定間隔でフレームを抽出して、
 * 抽出したフレームを元に Time-Based AGC のセットアップを行います。
 * OwLib_CaptureStart() の前に呼び出す必要があります。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param isLinearAGC [in]
 *		OwLib_Decode() において使用したいデコード形式。
 *		TRUE : 線形AGCを使用, FALSE : 非線形AGCを使用。
 * @param sampleFrameNumIntvl [in]
 *		フレームを抽出する間隔。
 * @param fromTimeOfs [in]
 *		計算を開始するオフセット時間。
 * @param toTimeOfs [in]
 *		計算を終了するオフセット時間。
 * @return
 *		エラーコード。
 *
 * @~english
 * Extracts frames from the raw-recording file at the specified interval and
 * setup Time-Based AGC  minimum and maximum temperatures of all the frames.
 * This function must be called before calling OwLib_CaptureStart().
 *
 * @param dev [in]
 *		The device handle.
 * @param isLinearAGC [in]
 *		The kind of decode type which will be passed to OwLib_Decode().
 *		TRUE : Use Linear AGC, FALSE : Use Non-Linear AGC.
 * @param sampleFrameNumIntvl [in]
 *		The interval number of frames for extracting.
 * @param fromTimeOfs [in]
 *		The time offset to start calculating.
 * @param toTimeOfs [in]
 *		The time offset to stop calculating.
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_FileTimeBasedAGCSetup(OwDev dev, BOOL isLinearAGC,
	UINT32 sampleFrameNumIntvl,
	UINT64 fromTimeOfs, UINT64 toTimeOfs);

/**
 * @~japanese
 * Time-Based AGC の有効・無効の状態を取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Gets whether or not Time-Based AGC is enabled.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The corresponding value.
 */
OWLIFTLIB_API BOOL
OwLib_GetTimeBasedAGC(OwDev dev);

/**
 * @~japanese
 * Time-Based AGC の有効・無効の状態を設定します。
 * TRUE を設定するときは、 OwLib_FileTimeBasedAGCSetup() を呼ぶ必要があります。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param enabled [in]
 *		該当する値。
 *
 * @~english
 * Sets whether or not Time-Based AGC is enabled.
 * OwLib_FileTimeBasedAGCSetup() must be called before setting TRUE.
 *
 * @param dev [in]
 *		The device handle.
 * @param enabled [in]
 *		The corresponding value.
 */
OWLIFTLIB_API void
OwLib_SetTimeBasedAGC(OwDev dev, BOOL enabled);


/**
 * @~japanese
 * 自動ゲイン制御のROI(Region of Interest)を設定します。
 * 興味のない領域にある高温または低温のピクセルが原因で全体の色合いが平坦になるときに利用します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @param mask [in]
 *		フレームの幅x高さの要素数を持つBOOLの2次元配列。
 *		ピクセルの位置に対応する値が TRUE のとき、そのピクセルは計算対象になります。
 *		全ての要素が FALSE のときはエラーコードが返されます。
 * @return
 *		エラーコード。
 *
 * @~english
 * Sets ROI for Auto Gain Control.
 * ROI is useful when colors look flat
 * because there are pixels of high or low temperature at uninsteresting area.
 *
 * @param dev [in]
 *		The device handle.
 * @param mask [in]
 *		Two dimensional BOOL array that has elements of frame width x frame height.
 *		When the mask value that position corresponds to the pixel's position is TRUE,
 *		the pixel is calculation target.
 *		If all elements are FALSEs, an error code is returned.
 *
 * @return
 *		The error code.
 */
OWLIFTLIB_API OwStatus_t
OwLib_SetAGCROIMask(OwDev dev, const BOOL *mask);


/**
 * @~japanese
 * Raw録画ファイルの録画中または再生中に発生したエラーを取得します。
 *
 * @param dev [in]
 *		デバイスハンドル。
 * @return
 *		該当する値。
 *
 * @~english
 * Returns the error occurred while recording or playing the raw-recording file.
 *
 * @param dev [in]
 *		The device handle.
 * @return
 *		The correspoinding value.
 */
OWLIFTLIB_API OwStatus_t
OwLib_GetFileError(OwDev dev);


/**
 * @~japanese
 * 温度データまたはRawデータをアスキー形式で出力します。
 * 
 * @param dev [in]
 *		デバイスハンドル。
 * @param filePath [in]
 *		出力先のファイルパス。NULL を指定すると標準出力に出力します。
 * @param buf [in]
 *		温度データまたはRawデータ。
 *		Rawデータは OWDECODETYPE_RAW_14 の変換結果を使用してください。
 * @param bufLen [in]
 *		buf のサイズ。
 * @param telemp [in]
 *		buf に対応する Telemetry Data。
 * @param dateTimep [in]
 *		出力する日付。NULLを指定すると現在時刻が使用されます。
 * @param tempOrRaw [in]
 *		TRUE : buf は温度データ, FALSE : buf はRawデータ。
 * @return
 *		エラーコード。
 *
 * @~english
 * Outputs temperature data or raw data as ascii format.
 *
 * @param dev [in]
 *		The device handle.
 * @param filePath [in]
 *		The file path to output. NULL means standard output.
 * @param buf [in]
 *		The temperature data or raw data.
 *		For a raw data, use a result of decoding by OwDecodeType.RAW_14.
 * @param bufLen [in]
 *		The size of 'buf'.
 * @param telemp [in]
 *		The telemetry data correponding to 'buf'.
 * @param dateTimep [in]
 *		The date time to output. NULL means a current date time, 
 * @param tempOrRaw [in]
 *		TRUE : 'buf' is temperature data, FALSE : 'buf' is raw data.
 * @return
 *		The error code.
 */
#ifdef _WIN32
OWLIFTLIB_API OwStatus_t
OwLib_DumpFrame(OwDev dev, const wchar_t *filePath, WORD *buf, UINT32 bufLen, const LepTelemetryDataLineA *telemp,
	const OwDateTime *dateTimep, BOOL tempOrRaw);
#else
OWLIFTLIB_API OwStatus_t
OwLib_DumpFrame(OwDev dev, const char *filePath, WORD *buf, UINT32 bufLen, const LepTelemetryDataLineA *telemp,
	const OwDateTime *dateTimep, BOOL tempOrRaw);
#endif

/**
 * @~japanese
 * 指定のデコード形式に対応するカラーテーブルに含まれる色の数を取得します。
 *
 * @param dtype
 *		デコード形式。
 * @return
 *		カラーテーブルに含まれる色の数。
 *
 * @~english
 * Returns the number of colors in the color table corresponding to the specified decode type.
 *
 * @param dtype
 *		Decode type.
 * @return
 *		The number of colors in the color table.
 */
OWLIFTLIB_API UINT32
OwLib_GetColorTableSize(OwDecodeType_t dtype);

/**
 * @~japanese
 * 指定のデコード形式に対応するカラーテーブルを取得します。
 * 各要素は24bit RGB値です。
 *
 * @param dtype
 *		デコード形式。
 * @param tab
 *		コピー先の領域。OwLib_GetColorTableSize() により取得した要素数を格納できるサイズであること。
 *
 * @~english
 * Returns the color table corresponding to the specified decode type.
 * The element is a value of 24bit RGB.
 *
 * @param dtype
 *		Decode type.
 * @param tab
 *		The destination area that can store elements of which
 *		number is gotten by OwLib_GetColorTableSize().
 */
void OwLib_GetColorTable(OwDecodeType_t dtype, DWORD *tab);


/* @} */

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus
